use strict;
use Test::More;
use DE_EPAGES::Core::API::Script qw( RunScript );
use DE_EPAGES::Database::API::Connection qw( ConnectTestSite DisconnectSite );
use DE_EPAGES::Object::API::Factory qw (ExistsObjectByPath LoadRootObject LoadObjectByPath);
use DE_EPAGES::Object::API::XML::ObjectImportHandler qw (ImportObjects);
use DE_EPAGES::WebInterface::API::FakeRequest;

my $ROOTPATH = '/Test';
local $ENV{'EPAGES_TESTSITE'} = 'Store' unless defined $ENV{'EPAGES_TESTSITE'};

sub Main() {
    ConnectTestSite();
    ok( Init(), 'initialization' );

    my $FakeRequester = DE_EPAGES::WebInterface::API::FakeRequest->new;

    my $Shop = LoadObjectByPath('/Test/TestShop/');
    ok( $Shop->alias eq 'TestShop', 'Shop loaded' );

    my $RootCategory = $Shop->get('Categories');
    my $Cat = $RootCategory->child('MinimalCategory');

    my $StoreURL = LoadRootObject()->get('WebUrl');

    # test homepage
    my $ShopURL = $Shop->get('WebUrlSSL');
    my $hAlternateLinks = visitSite($FakeRequester, $ShopURL);
    is( scalar keys %$hAlternateLinks, 4, "$ShopURL contains 4 alternate links." );
    is($hAlternateLinks->{'en'}, $StoreURL . '/en_GB/?ObjectPath=/Test/TestShop/Categories', "$ShopURL contains correct language link for language 'en'.");
    is($hAlternateLinks->{'de'}, $StoreURL . '/de_DE/?ObjectPath=/Test/TestShop/Categories', "$ShopURL contains correct language link for language 'de'.");
    is($hAlternateLinks->{'en-GB'}, $StoreURL . '/en_GB/?ObjectPath=/Test/TestShop/Categories', "$ShopURL contains correct alternate link for locale 'en-GB'.");
    is($hAlternateLinks->{'de-DE'}, $StoreURL . '/de_DE/?ObjectPath=/Test/TestShop/Categories', "$ShopURL contains correct alternate link for locale 'de-DE'.");

    # test category page
    my $CategoryURL = $Cat->get('WebUrlSSL');
    $hAlternateLinks = visitSite($FakeRequester, $CategoryURL);
    is( scalar keys %$hAlternateLinks, 4, "$CategoryURL contains 4 alternate links." );
    is($hAlternateLinks->{'en'}, $StoreURL . '/en_GB/?ObjectPath=/Test/TestShop/Categories/MinimalCategory', "$CategoryURL contains correct language link for language 'en'.");
    is($hAlternateLinks->{'de'}, $StoreURL . '/de_DE/?ObjectPath=/Test/TestShop/Categories/MinimalCategory', "$CategoryURL contains correct language link for language 'de'.");
    is($hAlternateLinks->{'en-GB'}, $StoreURL . '/en_GB/?ObjectPath=/Test/TestShop/Categories/MinimalCategory', "$CategoryURL contains correct alternate link for locale 'en-GB'.");
    is($hAlternateLinks->{'de-DE'}, $StoreURL . '/de_DE/?ObjectPath=/Test/TestShop/Categories/MinimalCategory', "$CategoryURL contains correct alternate link for locale 'de-DE'.");

    # test registration page
    my $RegistrationPage = $Shop->get('WebUrlSSL') . '&ViewAction=ViewRegistration';
    $hAlternateLinks = visitSite($FakeRequester, $RegistrationPage);
    is( scalar keys %$hAlternateLinks, 0, "$RegistrationPage shouldn't contain a language link." );
    $Shop->child('Languages')->child('de')->delete();

    # test homepage
    $hAlternateLinks = visitSite($FakeRequester, $ShopURL);
    is( scalar keys %$hAlternateLinks, 0, "$ShopURL shouldn't contain an alternate link." );

    # test category page
    $hAlternateLinks = visitSite($FakeRequester, $CategoryURL);
    is( scalar keys %$hAlternateLinks, 0, "$CategoryURL shouldn't contain an alternate link." );

    ok( Cleanup(), 'cleanup');
    DisconnectSite();
    done_testing();
}

sub visitSite() {
    my ($FakeRequester, $SiteURL) = @_;
    my $Response = $FakeRequester->get( $SiteURL );
    die "Can't get $SiteURL: ".$Response->status_line unless $Response->is_success;
    my $SiteContent = $Response->content;
    my $hAlternateLinks;
    while ($SiteContent =~ /<link rel="alternate" hreflang="(.*?)" href="(.*?)" >/g) {
        $hAlternateLinks->{$1} = $2;
    }
    return $hAlternateLinks;
}

sub Init() {
    return 1 if LoadRootObject()->existsChild('Test') && $ENV{'SKIP_CLEANUP'};
    Cleanup();
    LoadRootObject()->insertChild('Object', {'Alias'=>'Test'});
    ImportObjects('import.xml', $ROOTPATH);
    return 1;
}

sub Cleanup() {
    return 1 if $ENV{'SKIP_CLEANUP'};
    LoadObjectByPath($ROOTPATH)->delete if ExistsObjectByPath($ROOTPATH);
    return 1;
}

RunScript(Sub=>\&Main, Debug=>1);
