#========================================================================================
# package      DE_EPAGES::DHLIntraship::t::unit::AddressValidation
# state        public
#----------------------------------------------------------------------------------------
# description  class for unit test realted to AddressValidation.pm
#========================================================================================
package DE_EPAGES::DHLIntraship::t::unit::AddressValidation;
use base DE_EPAGES::Test::API::UnitTest::Test;

use strict;

use DE_EPAGES::DHLIntraship::API::AddressValidation;
use DE_EPAGES::DHLIntraship::API::Constants qw (
    MANDATORY_WARNING_KEYWORD
    MANDATORY_WARNING_STRING
    TOO_LONG_WARNING_STRING
    TOO_SHORT_WARNING_KEYWORD
    TOO_SHORT_WARNING_STRING
);
use DE_EPAGES::Test::Mock::MockifyEpagesObject;
use Test::More;
use Test::MockObject;
use Test::Exception;

use constant TEST_PHONE_TOO_LONG => '+40-00-123-456.6758-0344';
use constant TEST_PACKSTATION_TOO_SHORT => '22';
use constant TEST_VALID_HOUSE_NUMBER => 209;
use constant TEST_INVALID_HOUSE_NUMBER => 123456789012;

#sub Main {
sub test {
    my $self = shift;
    $self->test_Address_IsNotValid();
    $self->test_Address_HasWarningStates_positive();
    $self->test_Address_HasWarningStates_negative_wrong_parameters();
    $self->test_Address_HasWarningStates_negative_because_extra_parameters();
    $self->test_Address_Processable();
    $self->test_Address_Not_Processable_House_Number_Too_Long();
    $self->test_Address_Not_Processable_FullName_Not_Set();
    done_testing();
}

#========================================================================================
# Test methods
#========================================================================================
sub test_Address_IsNotValid {
    my $self = shift;
    my $subTestName = (caller(0))[3];

    my $ShopAddress = undef;
    ## no critic ( ProhibitComplexRegexes )
    throws_ok( sub {
                        DE_EPAGES::DHLIntraship::API::AddressValidation->new( $ShopAddress );
                   },
                   qr/PARAMETER_MISSING: Missing parameter AddressValidation needs valid address object in call to DE_EPAGES::DHLIntraship::API::AddressValidation::new/ms,
                   "$subTestName - is ShopAddress is a undefined"
    );
    $ShopAddress = $self->_createMock_WrongObject();
    throws_ok( sub {
                        DE_EPAGES::DHLIntraship::API::AddressValidation->new( $ShopAddress );
                   },
                   qr/WRONG_CLASS: AddressValidation needs valid address object is not an instance of DE_EPAGES::Address::API::Object::Address/ms,
                   "$subTestName - is ShopAddress is a blessed object but not an AddressObject"
    );
    ## use critic ( ProhibitComplexRegexes )
    $ShopAddress = $self->_createMock_AddressObject();
    isa_ok (
        DE_EPAGES::DHLIntraship::API::AddressValidation->new( $ShopAddress ),
        'DE_EPAGES::DHLIntraship::API::AddressValidation',
        "$subTestName - If the constuctor is called correctly it returns AddressValidation object"
    );
    return;
}

sub test_Address_HasWarningStates_positive {
    my $self = shift;
    my $subTestName = (caller(0))[3];

    ## no critic ( RequireNumberSeparators)
    my $BillingAddress = $self->_createMock_AddressObject();
    ## use critic (RequireNumberSeparators)

    my $ExtraAttributeDefinition = {};
    my $AddressValidation1 = DE_EPAGES::DHLIntraship::API::AddressValidation->new($BillingAddress, $ExtraAttributeDefinition);

    ok(!$AddressValidation1->hasWarningStates(), "$subTestName - address has no warning states (no extra attributes)");
    my $aWarningStates = $AddressValidation1->getStatesAsWarning();
    is (scalar @{$aWarningStates},0,"$subTestName - WarningStates array is empty");

    $ExtraAttributeDefinition = {
            'ExtraAttribute' => {'Mandatory'=> 0,'MinValue'=> 1, 'MaxValue'=>50},
            'ExtraAttributeMandatory' => {'Mandatory'=> 1,'MinValue'=> 1, 'MaxValue'=>50},
    } ;
    my $AddressValidation2 = DE_EPAGES::DHLIntraship::API::AddressValidation->new($BillingAddress, $ExtraAttributeDefinition);
    ok(!$AddressValidation2->hasWarningStates(), "$subTestName - address has no warning states (with extra attributes)");

    is($AddressValidation2->{'FirstName'}, 'Jon', "$subTestName - check a mandatory Attribute - Jon");
    is($AddressValidation2->{'Zipcode'}, 12345, "$subTestName - result for german zipcode - 12345"); ## no critic ( RequireNumberSeparators ProhibitMagicNumbers)
    is($AddressValidation2->{'ExtraAttribute'}, 'ExtraAttributeValue', "$subTestName - result for extra Attribute - ExtraAttributeValue");
    is($AddressValidation2->{'ExtraAttributeMandatory'}, 'ExtraAttributeValueMandatory', "$subTestName - result for Extra  mandatory Attribute - ExtraAttributeValueMandatory");
}

sub test_Address_HasWarningStates_negative_wrong_parameters {
    my $self = shift;
    my $subTestName = (caller(0))[3];

    my $BillingAddress = $self->_createMock_AddressObject(
            {
                'FullName' => '',
                'Phone'   => TEST_PHONE_TOO_LONG,
                'DHLPackstationId' => TEST_PACKSTATION_TOO_SHORT,
            }
    );
    my $ExtraAttributeDefinition = {};
    my $AddressValidation = DE_EPAGES::DHLIntraship::API::AddressValidation->new( $BillingAddress, $ExtraAttributeDefinition );

    ok($AddressValidation->hasWarningStates(), "$subTestName - address has warning states (standard fields do not fit specifications)");
    my $aWarningStates = $AddressValidation->getStatesAsWarning();
    is (scalar @{$aWarningStates}, 3, "$subTestName - getStatesAsWarning method returns proper array");
    my $FirstWarning = @{$aWarningStates}[0];
    is ( $FirstWarning->code(),
        'AddressValidationWarning',
        "$subTestName - First message code points to an address validation warning"
    );
    my $hFirstWarning = $FirstWarning->vars();
    is ( $hFirstWarning->{'attribute'},
        'DHLPackstationId',
        "$subTestName - First message attribute points to the Missing Packstation ID."
    );
    is ( $hFirstWarning->{'reason'},
        TOO_SHORT_WARNING_KEYWORD,
        "$subTestName - First message reason is the translation key for an \"attribute too short\" warning."
    );
    is_deeply($AddressValidation->{'WarningStates'},
            [
                { 'attribute' => 'DHLPackstationId', 'reason' => TOO_SHORT_WARNING_STRING},
                { 'attribute' => 'Phone', 'reason' => TOO_LONG_WARNING_STRING},
                { 'attribute' => 'FullName', 'reason' => MANDATORY_WARNING_STRING}
            ],
            "$subTestName - Status data about some fields too short, too long or not set."
    );
}

sub test_Address_HasWarningStates_negative_because_extra_parameters {
    my $self = shift;
    my $subTestName = (caller(0))[3];

    my $BillingAddress = $self->_createMock_AddressObject();
    my $ExtraAttributeDefinition = {
            'StreetOnly' => { 'Mandatory'=> 1,'MinValue'=> 20, 'MaxValue'=> 50 },
            'Company'    => { 'Mandatory'=> 1,'MinValue'=> 1, 'MaxValue'=> 50 }
    };
    my $AddressValidation = DE_EPAGES::DHLIntraship::API::AddressValidation->new( $BillingAddress, $ExtraAttributeDefinition );

    ok($AddressValidation->hasWarningStates(), "$subTestName - address has warning states (extra field specifications do not fit)");
    my $aWarningStates = $AddressValidation->getStatesAsWarning();

    my $FirstWarning = @{$aWarningStates}[0];
    is($FirstWarning->code(),
            'AddressValidationWarning',
            "$subTestName - First warning code is about an address validation warning."
    );
    is_deeply($FirstWarning->vars(),
            { 'attribute' => 'StreetOnly', 'reason' => TOO_SHORT_WARNING_KEYWORD},
            "$subTestName - First warning attribute and reason is about street-only field too short because extra specifications"
    );

    my $SecondWarning = @{$aWarningStates}[1];
    is_deeply($SecondWarning->vars(),
            { 'attribute' => 'Company', 'reason' => MANDATORY_WARNING_KEYWORD},
            "$subTestName - Second warning attribute and reason is about company field not set because extra specifications"
    );
}

sub test_Address_Processable {
    my $self = shift;
    my $subTestName = (caller(0))[3];

    my $BillingAddress = $self->_createMock_AddressObject();
    my $AddressValidation = DE_EPAGES::DHLIntraship::API::AddressValidation->new( $BillingAddress );
    my $aNotProcessableWarningStates = $AddressValidation->getNotProcessableWarningStates();

    is(scalar(@{$aNotProcessableWarningStates}), 0, "$subTestName - there are not warnings for not processable attributes.");
}

sub test_Address_Not_Processable_House_Number_Too_Long {
    my $self = shift;
    my $subTestName = (caller(0))[3];

    my $BillingAddress = $self->_createMock_AddressObject({'HouseNumber' => TEST_INVALID_HOUSE_NUMBER});
    my $AddressValidation = DE_EPAGES::DHLIntraship::API::AddressValidation->new( $BillingAddress );
    my $aNotProcessableWarningStates = $AddressValidation->getNotProcessableWarningStates();

    my $hFirstStatus = $aNotProcessableWarningStates->[0];
    is_deeply($hFirstStatus,
            { 'attribute' => 'HouseNumber', 'reason' => TOO_LONG_WARNING_STRING },
            "$subTestName - Not processable status attribute and reason is HouseNumber too long."
    );
}

sub test_Address_Not_Processable_FullName_Not_Set {
    my $self = shift;
    my $subTestName = (caller(0))[3];

    my $BillingAddress = $self->_createMock_AddressObject({'FullName' => ''});
    my $AddressValidation = DE_EPAGES::DHLIntraship::API::AddressValidation->new( $BillingAddress );
    my $aNotProcessableWarningStates = $AddressValidation->getNotProcessableWarningStates();

    my $hFirstStatus = $aNotProcessableWarningStates->[0];
    is_deeply($hFirstStatus,
            { 'attribute' => 'FullName', 'reason' => MANDATORY_WARNING_STRING },
            "$subTestName - Not processable status attribute and reason is full name mandatory but not set."
    );
}

#========================================================================================
# Mocking methods
#========================================================================================
sub _createMock_AddressObject {
    my $self = shift;
    my ($hAtributeParams) = @_;

    $hAtributeParams //= {};
    my $hAtributes = {
        'AddressExtension'        => $hAtributeParams->{'AddressExtension'} // 'TestAddressExtension',
        'Alias'                   => $hAtributeParams->{'Alias'} // 'BillingAddress ',
        'City'                    => $hAtributeParams->{'City'} // 'Hamburg',
        'Company'                 => $hAtributeParams->{'Company'} // '',
        'Country'                 => $hAtributeParams->{'Country'} // {'Code2'=> 'DE'},
        'DHLPackstationId'        => $hAtributeParams->{'DHLPackstationId'} // '123',
        'DHLPostNumber'           => $hAtributeParams->{'DHLPostNumber'} // '0123456789',
        'EMail'                   => $hAtributeParams->{'EMail'} // 'test@epages.com',
        'ExtraAttribute'          => $hAtributeParams->{'ExtraAttribute'} // 'ExtraAttributeValue',
        'ExtraAttributeMandatory' => $hAtributeParams->{'ExtraAttributeMandatory'} // 'ExtraAttributeValueMandatory',
        'FirstName'               => $hAtributeParams->{'FirstName'} // 'Jon',
        'FullName'                => $hAtributeParams->{'FullName'} // 'FullNameText',
        'HouseNumber'             => $hAtributeParams->{'HouseNumber'} // TEST_VALID_HOUSE_NUMBER,
        'IsPickupAddress'         => $hAtributeParams->{'IsPickupAddress'} // '1',
        'LastName'                => $hAtributeParams->{'LastName'} // 'Doe',
        'Phone'                   => $hAtributeParams->{'Phone'} // '',
        'Street2'                 => $hAtributeParams->{'Street2'} // 'c/o 123',
        'StreetOnly'              => $hAtributeParams->{'StreetOnly'} // 'Pilatuspool',
        'Zipcode'                 => $hAtributeParams->{'Zipcode'} // '12345',
    };
    my $MockifyAddressObject = DE_EPAGES::Test::Mock::MockifyEpagesObject->new('DE_EPAGES::Address::API::Object::Address',$hAtributes);

    return $MockifyAddressObject->getMockObject();
}

sub _createMock_WrongObject {
    my $self = shift;
    my ($hAtributes) = @_;

    $hAtributes //= {};
    my $MockifyObjectObject = DE_EPAGES::Test::Mock::MockifyEpagesObject->new('DE_EPAGES::Object::API::Object::Object',$hAtributes);

    return $MockifyObjectObject->getMockObject();
}

__PACKAGE__->runScript();