#========================================================================================
# package      DE_EPAGES::DHLIntraship::t::unit::ShippingLabelRequest_Address
# state        public
#----------------------------------------------------------------------------------------
# description  Unit test for Address module.
#========================================================================================
package DE_EPAGES::DHLIntraship::t::unit::ShippingLabelRequest_Address;
use base DE_EPAGES::Test::API::UnitTest::Test;

use DE_EPAGES::DHLIntraship::API::SOAP::ShippingLabelRequest::Address;

use DE_EPAGES::Test::Mock::Mockify;
use DE_EPAGES::Test::Mock::MockifyEpagesObject;
use Test::More;

use strict;

#use constant TEST_ADDRESS_COMPANY => 'My Company Name';
use constant TEST_ADDRESS_STREET_NAME => 'Diagonal';
use constant TEST_ADDRESS_STREET_NAME_TOO_LONG => 'street name that exceeds the maximum characters limit';
use constant TEST_ADDRESS_HOUSE_NUMBER => '435';
use constant TEST_ADDRESS_HOUSE_NUMBER_TOO_LONG => '43545678369521';
use constant TEST_ADDRESS_ADDRESS_ADDITION => '5th floor, 2nd door';
use constant TEST_ADDRESS_ZIP_CODE => '08036';
use constant TEST_ADDRESS_CITY => 'Barcelona';
use constant TEST_ADDRESS_CITY_TOO_LONG => 'city name that exceeds the maximum characters limit';
use constant TEST_ADDRESS_FULL_NAME => 'My Full Name';


#========================================================================================
sub test {
    my $self = shift;

    $self->address_shipper_assemble_positive_test();
    $self->address_shipper_assemble_address_fields_too_long_test();
    $self->address_shipper_assemble_subnodes_with_warnings_test();
    $self->address_receiver_assemble_positive_test();

    done_testing();
}

#========================================================================================
# Test methods
#========================================================================================
sub address_shipper_assemble_positive_test {
    my $self = shift;
    my $SubTestName = ( caller(0) )[3];

    my $mockedAddress = $self->_createAddress();
    my $AddressWithWarnings = 0;
    my $mockedAddressElement = $self->_createAddressElement($AddressWithWarnings);
    my $hAddress = $mockedAddressElement->assemble($mockedAddress);

    is_deeply($hAddress->{'nodeData'},
        { 'Address' => {
              'streetName' => TEST_ADDRESS_STREET_NAME,
              'streetNumber' => TEST_ADDRESS_HOUSE_NUMBER,
              'addressAddition' => TEST_ADDRESS_ADDRESS_ADDITION,
              'zip' => TEST_ADDRESS_ZIP_CODE,
              'city' => TEST_ADDRESS_CITY,
              'Origin' => {}
          }
        },
        "$SubTestName - address hash assembled"
    );

    is_deeply($hAddress->{'nodeWarnings'}, [], "$SubTestName - warnings array for Address is empty");
}

sub address_shipper_assemble_address_fields_too_long_test {
    my $self = shift;
    my $SubTestName = ( caller(0) )[3];

    my $mockedAddress = $self->_createAddress(
        {
            'StreetOnly' => TEST_ADDRESS_STREET_NAME_TOO_LONG,
            'HouseNumber' => TEST_ADDRESS_HOUSE_NUMBER_TOO_LONG,
            'City' => TEST_ADDRESS_CITY_TOO_LONG
        }
    );
    my $AddressWithWarnings = 0;
    my $mockedAddressElement = $self->_createAddressElement($AddressWithWarnings);
    my $hAddress = $mockedAddressElement->assemble($mockedAddress);
    my $StreetNameResult = $hAddress->{'nodeData'}->{'Address'}->{'streetName'};
    my $CityResult = $hAddress->{'nodeData'}->{'Address'}->{'city'};

    ok(length($StreetNameResult) > 0 && length($StreetNameResult) < length(TEST_ADDRESS_STREET_NAME_TOO_LONG),
        "$SubTestName - street name has been trimmed because it was too long"
    );

    ok(length($CityResult) > 0 &&  length($CityResult) < length(TEST_ADDRESS_CITY_TOO_LONG),
        "$SubTestName - city name has been trimmed because it was too long"
    );

    is_deeply($hAddress->{'nodeWarnings'},
        [
             { 'streetName' => 'StringTooLong' },
             { 'streetNumber' => 'StringTooLong' },
             { 'city' => 'StringTooLong' }
        ],
        "$SubTestName - warnings array for Address holds street and city messages about being too long"
    );
}

sub address_shipper_assemble_subnodes_with_warnings_test {
    my $self = shift;
    my $SubTestName = ( caller(0) )[3];

    my $mockedAddress = $self->_createAddress();
    my $AddressWithWarnings = 1;
    my $mockedAddressElement = $self->_createAddressElement($AddressWithWarnings);
    my $hAddress = $mockedAddressElement->assemble($mockedAddress);

    is_deeply($hAddress->{'nodeWarnings'},
        [ { 'subnode' => 'warning' } ],
        "$SubTestName - warnings array for Address coming from subnodes"
    );
}

sub address_receiver_assemble_positive_test {
    my $self = shift;
    my $SubTestName = ( caller(0) )[3];

    my $mockedAddress = $self->_createAddress();
    my $AddressWithWarnings = 0;
    my $mockedAddressElement = $self->_createAddressElement($AddressWithWarnings);
    my $hAddress = $mockedAddressElement->assemble($mockedAddress, TEST_ADDRESS_FULL_NAME);

    is_deeply($hAddress->{'nodeData'},
        { 'Address' => {
              'addressAddition' => TEST_ADDRESS_ADDRESS_ADDITION,
              'city' => TEST_ADDRESS_CITY,
              'name2' => TEST_ADDRESS_FULL_NAME,
              'streetName' => TEST_ADDRESS_STREET_NAME,
              'streetNumber' => TEST_ADDRESS_HOUSE_NUMBER,
              'zip' => TEST_ADDRESS_ZIP_CODE,
              'Origin' => {}
          }
        },
        "$SubTestName - address hash assembled"
    );

    is_deeply($hAddress->{'nodeWarnings'}, [], "$SubTestName - warnings array for Address is empty");
}

#========================================================================================
# Mocking methods
#========================================================================================
sub _createAddressElement {
    my $self = shift;
    my ($withWarning) = @_;

    my $aParameterList = [];

    my $Mockify = DE_EPAGES::Test::Mock::Mockify->new(
        'DE_EPAGES::DHLIntraship::API::SOAP::ShippingLabelRequest::Address',
        $aParameterList
    );

    my $hOriginReturn = {
        'nodeData' => { 'Origin' => {} },
        'nodeWarnings' => $withWarning? [{'subnode' => 'warning'}] : []
    };
    $Mockify->addMockWithReturnValueAndParameterCheck(
            '_assembleOrigin', $hOriginReturn, [ { 'object' => 'DE_EPAGES::Address::API::Object::Address' } ]
    );


    return $Mockify->getMockObject();
}

sub _createAddress {
    my $self = shift;
    my ($hAddress) = @_;

    $hAddress = {} unless defined($hAddress);

    my $hParameterList = {
        'StreetOnly' => $hAddress->{'StreetOnly'} // TEST_ADDRESS_STREET_NAME,
        'HouseNumber' => $hAddress->{'HouseNumber'} // TEST_ADDRESS_HOUSE_NUMBER,
        'Street2' => $hAddress->{'Street2'} // TEST_ADDRESS_ADDRESS_ADDITION,
        'Zipcode' => $hAddress->{'Zipcode'} // TEST_ADDRESS_ZIP_CODE,
        'City' => $hAddress->{'City'} // TEST_ADDRESS_CITY
    };

    my $MockAddress = DE_EPAGES::Test::Mock::MockifyEpagesObject->new(
            'DE_EPAGES::Address::API::Object::Address',
            $hParameterList
    );

    return $MockAddress->getMockObject();
}

__PACKAGE__->runScript;
