#========================================================================================
# package      DE_EPAGES::Forum::API::Cartridge
# base         DE_EPAGES::Design::API::DesignInstaller
# state        public
#----------------------------------------------------------------------------------------
# description  This is the main cartridge class for install/patch/uninstall.
#========================================================================================
package DE_EPAGES::Forum::API::Cartridge;
use base DE_EPAGES::ShopMailType::API::ShopMailTypeInstaller;

use strict;

#========================================================================================
# function     new
# state        private
#----------------------------------------------------------------------------------------
# syntax       $Installer = DE_EPAGES::Forum::API::Cartridge->new(%options);
#----------------------------------------------------------------------------------------
# description  Creates a new installer object of DE_EPAGES::Forum.
#----------------------------------------------------------------------------------------
# input        %options | options for un/install
#               <ul>
#                 <li>IsRecursive - recursive un/installation of required or dependent
#                     cartridges
#               </ul>
#               | hash
# return       $Installer | cartridge installer | object
#========================================================================================
sub new {
    my ($class, %options) = @_;

    my $self = __PACKAGE__->SUPER::new(
        %options,
        'CartridgeDirectory' => 'DE_EPAGES/Forum',
        'Version'            => '7.42.0',
        'Patches'            => ['6.15.1','6.15.2','6.15.3','6.15.4','6.16.0','6.16.1','6.16.2','6.16.3','6.17.0','6.17.1','6.17.2','6.17.3','6.17.4','6.17.5','6.17.6','6.17.7','6.17.8','6.17.9','6.17.9.1','6.17.9.2','6.17.9.3','6.17.10','6.17.11','6.17.12','6.17.12.1','6.17.12.2','6.17.13','6.17.13.1','6.17.13.2','6.17.14','6.17.14.1','6.17.15','6.17.15.1','6.17.16','6.17.17','6.17.17.1','6.17.18','6.17.19','6.17.20','6.17.20.1','6.17.21','6.17.21.1','6.17.22','6.17.23','6.17.24','6.17.24.1','6.17.25','6.17.26','6.17.26.1','6.17.27','6.17.27.1','6.17.28','6.17.28.1','6.17.29','6.17.29.1','6.17.30','6.17.31','6.17.32','6.17.33','6.17.33.1','6.17.34','6.17.34.1','6.17.35','6.17.36','6.17.36.1','6.17.37','6.17.38','6.17.38.1','6.17.39','6.17.39.1','6.17.40','6.17.40.1','6.17.41','6.17.41.1','6.17.42','6.17.43','6.17.43.1','6.17.44','6.17.45','6.17.45.1','6.17.46','6.17.47','6.17.47.1','6.17.48','6.17.48.1','6.17.49','6.17.49.1','6.17.50','6.17.51','6.17.51.1','6.17.52','6.17.53','7.0.0','7.1.0','7.2.0','7.3.0','7.4.0','7.5.0','7.6.0','7.7.0','7.8.0','7.9.0','7.10.0','7.11.0','7.12.0','7.13.0','7.14.0','7.15.0','7.16.0','7.17.0','7.18.0','7.19.0','7.20.0','7.21.0','7.22.0','7.23.0','7.24.0','7.25.0','7.26.0','7.27.0','7.28.0','7.29.0','7.30.0','7.31.0','7.32.0','7.33.0','7.34.0','7.35.0','7.36.0','7.37.0','7.38.0','7.39.0','7.40.0','7.41.0'],
    );
    return bless $self, $class;
}

#========================================================================================
# function     install
# state        private
#----------------------------------------------------------------------------------------
# syntax       $Installer->install;
#----------------------------------------------------------------------------------------
# description  Installs the cartridge.
#========================================================================================
sub install {
    my $self = shift;

    $self->startInstall;
    $self->SUPER::install;
    $self->finishInstall;
    return;
}

#========================================================================================
# function     uninstall
# state        private
#----------------------------------------------------------------------------------------
# syntax       $Installer->uninstall;
#----------------------------------------------------------------------------------------
# description  Uninstalls the cartridge.
#========================================================================================
sub uninstall {
    my $self = shift;

    $self->startUninstall;
    $self->SUPER::uninstall;
    $self->finishUninstall;
    return;
}

#========================================================================================
# function     patchDBStructure_6_17_0
# state        private
#----------------------------------------------------------------------------------------
# syntax       $Installer->patchDBStructure_6_17_0;
#----------------------------------------------------------------------------------------
# description  Patches cartridge db structure in active store (database connection)
#               from version '6.17.0' to version '6.17.1'.
#----------------------------------------------------------------------------------------
# return       $Version | version of cartridge after patch function, '6.17.1' | string
#========================================================================================
sub patchDBStructure_6_17_0 {
    my $self = shift;

    my $dbi = $self->getCurrentDBHandle();

    if($dbi->driverName() eq 'mysql'){

        #remove old version of view
        if( $dbi->existsView('v_forumthread') ) {
            $self->logInfo("drop old version of view 'v_forumthread'");
            $dbi->dropView('v_forumthread');
        }

        #add new version of view
        $self->logInfo("add new version of view 'v_forumthread'");
        $dbi->do(<<'END_SQL');
CREATE VIEW v_forumthread AS
SELECT ft.forumid, ft.forumpostid, ft.subject, MAX(fp.forumpostid) AS lastpostid, COUNT(fp.forumpostid) AS postcount
FROM forumpost ft INNER JOIN forumpost fp ON fp.threadid = ft.forumpostid
WHERE ft.forumpostid = ft.threadid GROUP BY ft.forumid, ft.forumpostid, ft.subject
END_SQL

    }

    return '6.17.1';
}

#========================================================================================
# function     patchDBStructure_6_17_4
# state        private
#----------------------------------------------------------------------------------------
# syntax       $Installer->patchDBStructure_6_17_4;
#----------------------------------------------------------------------------------------
# description  Patches cartridge db structure in active store (database connection)
#               from version '6.17.4' to version '6.17.5'.
#----------------------------------------------------------------------------------------
# return       $Version | version of cartridge after patch function, '6.17.5' | string
#========================================================================================
sub patchDBStructure_6_17_4 {
    my $self = shift;
    #dont use API function, to avoid inconistencies between perl modules and database content

    return '6.17.5';
}

#========================================================================================
# function     patchDBStructure_6_17_5
# state        private
#----------------------------------------------------------------------------------------
# syntax       $Installer->patchDBStructure_6_17_5;
#----------------------------------------------------------------------------------------
# description  Patches cartridge db structure in active store (database connection)
#               from version '6.17.5' to version '6.17.6'.
#----------------------------------------------------------------------------------------
# return       $Version | version of cartridge after patch function, '6.17.6' | string
#========================================================================================
sub patchDBStructure_6_17_5 {
    my $self = shift;
    #dont use API function, to avoid inconistencies between perl modules and database content

    my $dbi = $self->getCurrentDBHandle();

    if($dbi->driverName() eq 'mysql'){

        #remove old version of view
        if( $dbi->existsView('v_visibleforumthread') ) {
            $self->logInfo("drop old version of view 'v_visibleforumthread'");
            $dbi->dropView('v_visibleforumthread');
        }

        #add new version of view
        $self->logInfo("add new version of view 'v_visibleforumthread'");
        $dbi->do(<<'END_SQL');
CREATE VIEW v_visibleforumthread AS
SELECT ft.forumid, ft.forumpostid, ft.subject, MAX(fp.forumpostid) AS lastpostid, COUNT(fp.forumpostid) AS postcount
FROM forumpost ft INNER JOIN forumpost fp ON fp.threadid = ft.forumpostid
WHERE ft.forumpostid = ft.threadid AND ft.isvisible = 1 AND fp.isvisible = 1 GROUP BY ft.forumid, ft.forumpostid, ft.subject
END_SQL

    }

    return '6.17.6';
}

1;
