use strict;
use Test::More;
use DE_EPAGES::Core::API::Script qw( RunScript );
use DE_EPAGES::Database::API::Connection qw( ConnectTestSite DisconnectSite );
use DE_EPAGES::Object::API::Factory qw (ExistsObjectByPath LoadRootObject LoadObjectByPath);
use DE_EPAGES::Object::API::XML::ObjectImportHandler qw (ImportObjects);
use DE_EPAGES::WebInterface::API::FakeRequest;

my $ROOTPATH = '/Test';
local $ENV{'EPAGES_TESTSITE'} = 'Store' unless defined $ENV{'EPAGES_TESTSITE'};

sub Main() {
    ConnectTestSite();
    ok( Init(), 'initialization' );

    local $DE_EPAGES::Object::API::Language::LANGUAGEID = 1;
    my $FakeRequester = DE_EPAGES::WebInterface::API::FakeRequest->new;

    my $Shop = LoadObjectByPath('/Test/TestShop/');
    ok( $Shop->alias eq 'TestShop', 'Shop loaded' );
    $Shop->set({'DomainName' => 'example.com'});

    ok ( 2 == scalar @{$Shop->languages}, 'Shop language count' );

    my $RootCategory = $Shop->get('Categories');
    my $Blog = $RootCategory->child('Blog');

    # used hash and loop to make it easier to expand test case to other content objects
    my $BlogURL = $Blog->get('WebUrl');

    my $hAlternateLinks = languageLink($FakeRequester, $BlogURL);
    is( scalar keys %$hAlternateLinks, 4, "$BlogURL contains 4 alternate links." );
    is($hAlternateLinks->{'en'}, 'http://example.com/BLOG', "$BlogURL contains correct language link for language 'en'.");
    is($hAlternateLinks->{'de'}, 'http://example.com/epages/Store.sf/de_DE/?ObjectPath=/Test/TestShop/Categories/Blog', "$BlogURL contains correct language link for language 'de'.");
    is($hAlternateLinks->{'en-GB'}, 'http://example.com/BLOG', "$BlogURL contains correct alternate link for locale 'en-GB'.");
    is($hAlternateLinks->{'de-DE'}, 'http://example.com/epages/Store.sf/de_DE/?ObjectPath=/Test/TestShop/Categories/Blog', "$BlogURL contains correct alternate link for locale 'de-DE'.");

    my $Can = canonicalTag($FakeRequester, $BlogURL);
    is( $Can, 'http://example.com/BLOG', "$BlogURL: check canonical tag." );

    my $BlogURLPage2 = $BlogURL.'&Page=2';
    $Can = canonicalTag($FakeRequester, $BlogURLPage2);
    ok( $Can eq '', 'check that there is no canonical tag.' );
    $hAlternateLinks = languageLink($FakeRequester, $BlogURLPage2);
    is( scalar keys %$hAlternateLinks, 0, 'check that there are no alternate links.' );

    # ok( Cleanup(), 'cleanup');
    DisconnectSite();
    done_testing();
}

sub languageLink() {
    my ($FakeRequester, $SiteURL) = @_;
    my $Response = $FakeRequester->get( $SiteURL );
    die "Can't get $SiteURL: ".$Response->status_line unless $Response->is_success;
    my $SiteContent = $Response->content;
    my $hAlternateLinks;
    while ($SiteContent =~ /<link rel="alternate" hreflang="(.*?)" href="(.*?)" >/g) {
        $hAlternateLinks->{$1} = $2;
    }
    return $hAlternateLinks;
}


sub canonicalTag() {
    my ($FakeRequester, $SiteURL) = @_;
    my $Response = $FakeRequester->get( $SiteURL );
    die "Can't get $SiteURL: ".$Response->status_line unless $Response->is_success;
    my $SiteContent = $Response->content;
    my ($Href) = ($SiteContent =~ /<link rel="canonical" href="(.*?)" \/>/);
    return $Href;
}

sub Init() {
    return 1 if LoadRootObject()->existsChild('Test') && $ENV{'SKIP_CLEANUP'};
    Cleanup();
    LoadRootObject()->insertChild('Object', {'Alias'=>'Test'});
    ImportObjects('import.xml', $ROOTPATH);
    return 1;
}

sub Cleanup() {
    return 1 if $ENV{'SKIP_CLEANUP'};
    LoadObjectByPath($ROOTPATH)->delete if ExistsObjectByPath($ROOTPATH);
    return 1;
}

RunScript(Sub=>\&Main, Debug=>1);