#========================================================================================
# package      DE_EPAGES::Object::API::Table::UnitOfMeasurement
# state        public
# description  interface for table UnitOfMeasurement
#========================================================================================
package DE_EPAGES::Object::API::Table::UnitOfMeasurement;
use base qw( Exporter );

use strict;
our @EXPORT_OK = qw(
    GetAllPKeyUnitOfMeasurement
    GetAllInfoUnitOfMeasurement
    GetInfoUnitOfMeasurement
    ExistsUnitOfMeasurement
    GetPKeyUnitOfMeasurementByCode
    GetInfoUnitOfMeasurementByCode
    GetPKeyUnitOfMeasurementByAbbreviation
    GetInfoUnitOfMeasurementByAbbreviation
);

use DE_EPAGES::Core::API::Error         qw ( Error );
use DE_EPAGES::Core::API::TestParameter qw ( TestString TestInteger TestFloat TestDateTime ); ## no critic qw(RequireCorrectImports)
use DE_EPAGES::Object::DAL::UnitOfMeasurement;

#ERRORDEFINITION
sub ErrorDefinition {
    return {
        'NoResults_UnitOfMeasurement' => 'UnitOfMeasurement doesn\'t exist (ObjectID=#ObjectID)',
    };
}
#ERRORDEFINITION END

#========================================================================================
# function     GetAllPKeyUnitOfMeasurement
#----------------------------------------------------------------------------------------
# syntax       $ahMoneyCurrencyAttributeIDs = GetAllPKeyUnitOfMeasurement();
#----------------------------------------------------------------------------------------
# description  Returns all UnitOfMeasurement identifiers.
#----------------------------------------------------------------------------------------
# return       $ahUnitOfMeasurementIDs | with following keys:
#               <ul>
#                   <li>UnitID - object id (part of primary key)- int
#               </ul>| reference to array of hashes
#========================================================================================
sub GetAllPKeyUnitOfMeasurement () { DAL_GetAllPKeyUnitOfMeasurement(); }

#========================================================================================
# function     GetAllInfoUnitOfMeasurement
#----------------------------------------------------------------------------------------
# syntax       $ahUnitOfMeasurement = GetAllInfoUnitOfMeasurement();
#----------------------------------------------------------------------------------------
# description  Returns all objects of unitofmeasurement.
#----------------------------------------------------------------------------------------
# return       $ahUnitOfMeasurement | with following keys:
#               <ul>
#                   <li>UnitID       - unit id           (primary key)- int
#                   <li>Type         - type of measurement            - string
#                   <li>Factor       - conversion factor to base unit - decimal
#                   <li>Abbreviation - representation symbol          - char(50)
#                   <li>Code         - UN/ECE recommendation 20 code  - char(3)
#               </ul>| reference to array of hashes
#========================================================================================
sub GetAllInfoUnitOfMeasurement () { DAL_GetAllInfoUnitOfMeasurement(); }

#========================================================================================
# function     GetInfoUnitOfMeasurement
#----------------------------------------------------------------------------------------
# syntax       $hUnitOfMeasurement = GetInfoUnitOfMeasurement($UnitID);
#----------------------------------------------------------------------------------------
# description  Returns the UnitOfMeasurement by identifier.
#----------------------------------------------------------------------------------------
# input        $UnitID | unit id (primary key) | int
# return       $hUnitOfMeasurement | with following keys:
#               <ul>
#                   <li>UnitID       - unit id           (primary key)- int
#                   <li>Type         - type of measurement            - string
#                   <li>Factor       - conversion factor to base unit - decimal
#                   <li>Abbreviation - representation symbol          - char(50)
#                   <li>Code         - UN/ECE recommendation 20 code  - char(3)
#               </ul>| reference to hash
#========================================================================================
sub GetInfoUnitOfMeasurement ($) {
    my ($UnitID) = @_;
    TestInteger('UnitID',$UnitID);
    my $hUnitOfMeasurement = DAL_GetInfoUnitOfMeasurement($UnitID);
    Error('NoResults_UnitOfMeasurement', { 'UnitID' => $UnitID }) unless defined $hUnitOfMeasurement;
    return $hUnitOfMeasurement;
}

#========================================================================================
# function     ExistsUnitOfMeasurement
#----------------------------------------------------------------------------------------
# syntax       $exists = ExistsUnitOfMeasurement($UnitID);
#----------------------------------------------------------------------------------------
# description  Returns true if the UnitOfMeasurement exists.
#----------------------------------------------------------------------------------------
# input        $UnitID | unit id (primary key) | int
# return       $exists | 1/0  the UnitOfMeasurement exists (yes/no) | boolean
#========================================================================================
sub ExistsUnitOfMeasurement ($) {
    my ($UnitID) = @_;
    TestInteger('UnitID',$UnitID);
    my $hUnitOfMeasurement = DAL_GetInfoUnitOfMeasurement($UnitID);
    return defined $hUnitOfMeasurement ? 1 : 0;
}

#========================================================================================
# function     GetPKeyUnitOfMeasurementByCode
#----------------------------------------------------------------------------------------
# syntax       $ahKeys = GetPKeyUnitOfMeasurementByCode($Code);
#----------------------------------------------------------------------------------------
# description  Returns primary key of UnitOfMeasurement by Code element.
#----------------------------------------------------------------------------------------
# input        $Code | code id | string
# return       $ahKeys | with following keys:
#               <ul>
#                   <li>UnitID - unit id (primary key)- int
#               </ul> | reference to array of hashes
#========================================================================================
sub GetPKeyUnitOfMeasurementByCode ($) {
    my ($Code) = @_;
    TestInteger('Code',$Code);
    return DAL_GetPKeyUnitOfMeasurementByCode($Code);
}

#========================================================================================
# function     GetInfoUnitOfMeasurementByCode
#----------------------------------------------------------------------------------------
# syntax       $ahUnitOfMeasurement = GetInfoUnitOfMeasurementByCode($Code);
#----------------------------------------------------------------------------------------
# description  Returns objects of UnitOfMeasurement by Code element.
#----------------------------------------------------------------------------------------
# input        $Code | code id | string
# return       $ahUnitOfMeasurement | with following keys:
#               <ul>
#                   <li>UnitID       - unit id           (primary key)- int
#                   <li>Type         - type of measurement            - string
#                   <li>Factor       - conversion factor to base unit - decimal
#                   <li>Abbreviation - representation symbol          - char(50)
#                   <li>Code         - UN/ECE recommendation 20 code  - char(3)
#               </ul> | reference to array of hashes
#========================================================================================
sub GetInfoUnitOfMeasurementByCode ($) {
    my ($Code) = @_;
    TestInteger('Code',$Code);
    return DAL_GetInfoUnitOfMeasurementByCode($Code);
}

#========================================================================================
# function     GetPKeyUnitOfMeasurementByAbbreviation
#----------------------------------------------------------------------------------------
# syntax       $ahKeys = GetPKeyUnitOfMeasurementByAbbreviation($Abbreviation);
#----------------------------------------------------------------------------------------
# description  Returns primary key of UnitOfMeasurement by Abbreviation element.
#----------------------------------------------------------------------------------------
# input        $Abbreviation | abbreviation id | string
# return       $ahKeys | with following keys:
#               <ul>
#                   <li>UnitID - unit id (primary key)- int
#               </ul> | reference to array of hashes
#========================================================================================
sub GetPKeyUnitOfMeasurementByAbbreviation ($) {
    my ($Abbreviation) = @_;
    TestInteger('Abbreviation',$Abbreviation);
    return DAL_GetPKeyUnitOfMeasurementByAbbreviation($Abbreviation);
}

#========================================================================================
# function     GetInfoUnitOfMeasurementByAbbreviation
#----------------------------------------------------------------------------------------
# syntax       $ahUnitOfMeasurement = GetInfoUnitOfMeasurementByAbbreviation($Abbreviation);
#----------------------------------------------------------------------------------------
# description  Returns objects of UnitOfMeasurement by Abbreviation element.
#----------------------------------------------------------------------------------------
# input        $Abbreviation | abbreviation id | string
# return       $ahUnitOfMeasurement | with following keys:
#               <ul>
#                   <li>UnitID       - unit id           (primary key)- int
#                   <li>Type         - type of measurement            - string
#                   <li>Factor       - conversion factor to base unit - decimal
#                   <li>Abbreviation - representation symbol          - char(50)
#                   <li>Code         - UN/ECE recommendation 20 code  - char(3)
#               </ul> | reference to array of hashes
#========================================================================================
sub GetInfoUnitOfMeasurementByAbbreviation ($) {
    my ($Abbreviation) = @_;
    TestInteger('Abbreviation',$Abbreviation);
    return DAL_GetInfoUnitOfMeasurementByAbbreviation($Abbreviation);
}

1;

