#========================================================================================
# package      DE_EPAGES::Object::DAL::SQL::UnitOfMeasurement
# state        private
# description  Generic SQL database interface for tables:
#               <ul>
#                   <li>unitofmeasurement
#               </ul>
#========================================================================================
package DE_EPAGES::Object::DAL::SQL::UnitOfMeasurement;

use strict;
use DE_EPAGES::Core::API::Error qw (Error);

## no critic qw(ProhibitLongLines)

my %TABLE_DEF = (
    'UnitOfMeasurement' => {
        'UnitID' => ['unitid','Integer'],
        'Type' => ['type','String'],
        'Factor' => ['factor','Float'],
        'Abbreviation' => ['abbreviation','String'],
        'Code' => ['code','String'],
    },
);

sub ErrorDefinition {
    return {
        'TableNotExists'  => 'table #Table does not exist at package DE_EPAGES::Object::DAL::SQL::UnitOfMeasurement',
        'ColumnNotExists' => 'column #Column does not exist at table #Table at package DE_EPAGES::Object::DAL::SQL::UnitOfMeasurement',
    };
}

sub TableDef {
    my $self = shift;
    my ($Table, $Column) = @_;
    Error('TableNotExists',  {'Table'=>$Table, 'Column'=>$Column }) unless exists $TABLE_DEF{$Table};
    Error('ColumnNotExists', {'Table'=>$Table, 'Column'=>$Column }) unless exists $TABLE_DEF{$Table}{$Column};
    return $TABLE_DEF{$Table}{$Column}[0];
}

sub TableType {
    my $self = shift;
    my ($Table, $Column) = @_;
    Error('TableNotExists',  {'Table'=>$Table, 'Column'=>$Column }) unless exists $TABLE_DEF{$Table};
    Error('ColumnNotExists', {'Table'=>$Table, 'Column'=>$Column }) unless exists $TABLE_DEF{$Table}{$Column};
    return $TABLE_DEF{$Table}{$Column}[1];
}

#========================================================================================
# function     GetAllPKeyUnitOfMeasurement
#----------------------------------------------------------------------------------------
# syntax       $ahUnitOfMeasurementIDs = $Package->GetAllPKeyUnitOfMeasurement($dbi);
#----------------------------------------------------------------------------------------
# description  Returns all primary keys of table unitofmeasurement.
#----------------------------------------------------------------------------------------
# input        $dbi | database handle | DE_EPAGES::Database::API::DBI
# return       $ahUnitOfMeasurementIDs | with following keys:
#               <ul>
#                   <li>UnitID - unit id (primary key)- int
#               </ul>| reference to array of hashes
#========================================================================================
sub GetAllPKeyUnitOfMeasurement {
    my $self = shift;
    my ($dbi) = @_;
    my $dbResult = $dbi->execute('SELECT unitid FROM unitofmeasurement', [], ['Integer'], [qw(UnitID)]);
    return $dbResult;
}

#========================================================================================
# function     GetAllInfoUnitOfMeasurement
#----------------------------------------------------------------------------------------
# syntax       $ahUnitOfMeasurementIDs = $Package->GetAllInfoUnitOfMeasurement($dbi);
#----------------------------------------------------------------------------------------
# description  Returns all rows of table unitofmeasurement.
#----------------------------------------------------------------------------------------
# input        $dbi | database handle | DE_EPAGES::Database::API::DBI
# return       $ahUnitOfMeasurementIDs | with following keys:
#               <ul>
#                   <li>UnitID       - unit id           (primary key)- int
#                   <li>Type         - type of measurement            - string
#                   <li>Factor       - conversion factor to base unit - decimal
#                   <li>Abbreviation - representation symbol          - char(50)
#                   <li>Code         - UN/ECE recommendation 20 code  - char(3)
#               </ul>| reference to array of hashes
#========================================================================================
sub GetAllInfoUnitOfMeasurement {
    my $self = shift;
    my ($dbi) = @_;
    return $dbi->execute('SELECT unitid, type, factor, abbreviation, code FROM unitofmeasurement', [],
        ['Integer', 'String', 'Float', 'String', 'String'],
        [qw(UnitID Type Factor Abbreviation Code)]
    );
}

#========================================================================================
# function     GetInfoUnitOfMeasurement
#----------------------------------------------------------------------------------------
# syntax       $hResult = $Package->GetInfoUnitOfMeasurement($dbi, $UnitID);
#----------------------------------------------------------------------------------------
# description  Returns a single row by primary key.
#----------------------------------------------------------------------------------------
# input        $dbi | database handle | DE_EPAGES::Database::API::DBI
# input        $UnitID      | unit id            (primary key) | int
# return       $hResult | with following keys:
#               <ul>
#                   <li>UnitID       - unit id           (primary key)- int
#                   <li>Type         - type of measurement            - string
#                   <li>Factor       - conversion factor to base unit - decimal
#                   <li>Abbreviation - representation symbol          - char(50)
#                   <li>Code         - UN/ECE recommendation 20 code  - char(3)
#               </ul>| reference to hash
#========================================================================================
sub GetInfoUnitOfMeasurement {
    my $self = shift;
    my ($dbi, $UnitID) = @_;
    return $dbi->execute('SELECT unitid, type, factor, abbreviation, code FROM unitofmeasurement WHERE unitid=?',
        [$UnitID],
        ['Integer', 'String', 'Float', 'String', 'String'],
        [qw(UnitID Type Factor Abbreviation Code)]
    )->[0];
}

#========================================================================================
# function     GetPKeyUnitOfMeasurementByCode
#----------------------------------------------------------------------------------------
# syntax       $ahResult = $Package->GetPKeyUnitOfMeasurementByCode($dbi, $Code);
#----------------------------------------------------------------------------------------
# description  Returns primary keys of table unitofmeasurement by code.
#----------------------------------------------------------------------------------------
# input        $dbi | database handle | DE_EPAGES::Database::API::DBI
# input        $Code | unit code (optional) | string
# return       $ahResult | with following keys:
#               <ul>
#                   <li>UnitID      - unit id            (primary key)- int
#               </ul>|reference to array of hashes
#========================================================================================
sub GetPKeyUnitOfMeasurementByCode {
    my $self = shift;
    my ($dbi, $Code) = @_;

    my @Parameter = ($Code);
    my $sql = 'SELECT unitid FROM unitofmeasurement WHERE code = ?';
    my $dbResult = $dbi->execute($sql, \@Parameter, ['Integer'], [qw(UnitID)]);
    return $dbResult;
}

#========================================================================================
# function     GetInfoUnitOfMeasurementByCode
#----------------------------------------------------------------------------------------
# syntax       $ahResult = $Package->GetInfoUnitOfMeasurementByCode($dbi, $Code);
#----------------------------------------------------------------------------------------
# description  Returns full rows of table unitofmeasurement by code.
#----------------------------------------------------------------------------------------
# input        $dbi | database handle | DE_EPAGES::Database::API::DBI
# input        $Code | unit code (optional) | string
# return       $ahResult | with following keys:
#               <ul>
#                   <li>UnitID       - unit id           (primary key)- int
#                   <li>Type         - type of measurement            - string
#                   <li>Factor       - conversion factor to base unit - decimal
#                   <li>Abbreviation - representation symbol          - char(50)
#                   <li>Code         - UN/ECE recommendation 20 code  - char(3)
#               </ul>|reference to array of hashes
#========================================================================================
sub GetInfoUnitOfMeasurementByCode {
    my $self = shift;
    my ($dbi, $Code) = @_;

    return $dbi->execute('SELECT unitid, type, factor, abbreviation, code FROM unitofmeasurement WHERE code = ?',
        [$Code],
        ['Integer', 'String', 'Float', 'String', 'String'],
        [qw(UnitID Type Factor Abbreviation Code)]
    );
}

#========================================================================================
# function     GetPKeyUnitOfMeasurementByAbbreviation
#----------------------------------------------------------------------------------------
# syntax       $ahResult = $Package->GetPKeyUnitOfMeasurementByAbbreviation($dbi, $Abbreviation);
#----------------------------------------------------------------------------------------
# description  Returns primary keys of table unitofmeasurement by abbreviation.
#----------------------------------------------------------------------------------------
# input        $dbi | database handle | DE_EPAGES::Database::API::DBI
# input        $Abbreviation | abbreviation unit code (optional) | string
# return       $ahResult | with following keys:
#               <ul>
#                   <li>UnitID      - unit id            (primary key)- int
#               </ul>|reference to array of hashes
#========================================================================================
sub GetPKeyUnitOfMeasurementByAbbreviation {
    my $self = shift;
    my ($dbi, $Abbreviation) = @_;

    my @Parameter = ($Abbreviation);
    my $sql = 'SELECT unitid FROM unitofmeasurement WHERE abbreviation = ?';
    my $dbResult = $dbi->execute($sql, \@Parameter, ['Integer'], [qw(UnitID)]);
    return $dbResult;
}

#========================================================================================
# function     GetInfoUnitOfMeasurementByAbbreviation
#----------------------------------------------------------------------------------------
# syntax       $ahResult = $Package->GetInfoUnitOfMeasurementByAbbreviation($dbi, $Abbreviation);
#----------------------------------------------------------------------------------------
# description  Returns full rows of table unitofmeasurement by abbreviation.
#----------------------------------------------------------------------------------------
# input        $dbi | database handle | DE_EPAGES::Database::API::DBI
# input        $Abbreviation | abbreviation unit code (optional) | string
# return       $ahResult | with following keys:
#               <ul>
#                   <li>UnitID       - unit id           (primary key)- int
#                   <li>Type         - type of measurement            - string
#                   <li>Factor       - conversion factor to base unit - decimal
#                   <li>Abbreviation - representation symbol          - char(50)
#                   <li>Code         - UN/ECE recommendation 20 code  - char(3)
#               </ul>|reference to array of hashes
#========================================================================================
sub GetInfoUnitOfMeasurementByAbbreviation {
    my $self = shift;
    my ($dbi, $Abbreviation) = @_;

    return $dbi->execute('SELECT unitid, type, factor, abbreviation, code FROM unitofmeasurement WHERE abbreviation = ?',
        [$Abbreviation],
        ['Integer', 'String', 'Float', 'String', 'String'],
        [qw(UnitID Type Factor Abbreviation Code)]
    );
}

1;
