#========================================================================================
# package      DE_EPAGES::Object::DAL::UnitOfMeasurement
# state        private
# description  database interface for tables:
#               <ul>
#                   <li>unitofmeasurement
#               </ul>
#========================================================================================
package DE_EPAGES::Object::DAL::UnitOfMeasurement;
use base qw (Exporter);

use strict;
our @EXPORT = qw (
    DAL_GetAllPKeyUnitOfMeasurement
    DAL_GetAllInfoUnitOfMeasurement
    DAL_GetInfoUnitOfMeasurement
    DAL_GetPKeyUnitOfMeasurementByCode
    DAL_GetInfoUnitOfMeasurementByCode
    DAL_GetPKeyUnitOfMeasurementByAbbreviation
    DAL_GetInfoUnitOfMeasurementByAbbreviation
);

use DE_EPAGES::Core::API::Log qw ( GetLog );
use DE_EPAGES::Core::API::PerlTools qw ( UsePackage );
use DE_EPAGES::Database::API::Connection qw (  GetCurrentDBHandle );
use DE_EPAGES::Database::API::DAL;
use DE_EPAGES::DataCache::API::Cache qw ( GetCache GetCacheSize );

my $DAL_PACKAGE = 'DE_EPAGES::Database::API::DAL';
my $DO_CACHE = 1;

sub _CallDriver {
    my ($FunctionName, @Parameter) = @_;
    my $dbi = GetCurrentDBHandle();
    my $Package = 'DE_EPAGES::Object::DAL::'.$dbi->driverName.'::UnitOfMeasurement';
    UsePackage($Package);
    GetLog->debug("call $Package->$FunctionName()");
    return $Package->$FunctionName($dbi, @Parameter);
}

sub _DoFillInfoCache ($$) {
    my ($CacheName, $CountRows) = @_;
    my $dbi = GetCurrentDBHandle();
    my $CacheSize = GetCacheSize($dbi->cacheKey(), $CacheName);
    return $CountRows < $CacheSize ? 1 : 0;
}

#========================================================================================
# function     DAL_GetAllPKeyUnitOfMeasurement
#----------------------------------------------------------------------------------------
# syntax       $rahUnitOfMeasurementIDs = DAL_GetAllPKeyUnitOfMeasurement();
#----------------------------------------------------------------------------------------
# description  returns all primary keys of table unitofmeasurement
#----------------------------------------------------------------------------------------
# return       $rahUnitOfMeasurementIDs | with following keys:
#               <ul>
#                   <li>UnitID - object id (part of primary key)- int
#               </ul>| reference to array of hashes
#========================================================================================
sub DAL_GetAllPKeyUnitOfMeasurement () {
    return _CallDriver('GetAllPKeyUnitOfMeasurement') unless $DO_CACHE;

    my $dbi = GetCurrentDBHandle();
    my $CacheRef = GetCache($dbi->cacheKey(),'AllUnitOfMeasurement');
    $CacheRef->{'1'} = _CallDriver('GetAllPKeyUnitOfMeasurement') unless exists $CacheRef->{'1'};
    return [ @{$CacheRef->{'1'}} ];
}

#========================================================================================
# function     DAL_GetAllInfoUnitOfMeasurement
#----------------------------------------------------------------------------------------
# syntax       $rahUnitOfMeasurementIDs = DAL_GetAllInfoUnitOfMeasurement();
#----------------------------------------------------------------------------------------
# description  returns all rows of table unitofmeasurement
#----------------------------------------------------------------------------------------
# return       $rahUnitOfMeasurementIDs | with following keys:
#               <ul>
#                   <li>UnitID       - unit id           (primary key)- int
#                   <li>Type         - type of measurement            - string
#                   <li>Factor       - conversion factor to base unit - decimal
#                   <li>Abbreviation - representation symbol          - char(50)
#                   <li>Code         - UN/ECE recommendation 20 code  - char(3)
#               </ul>| reference to array of hashes
#========================================================================================
sub DAL_GetAllInfoUnitOfMeasurement ();
sub DAL_GetAllInfoUnitOfMeasurement () {
    return _CallDriver('GetAllInfoUnitOfMeasurement') unless $DO_CACHE;

    my $dbi = GetCurrentDBHandle();
    my $CacheRef = GetCache($dbi->cacheKey(),'AllUnitOfMeasurement');
    if (not exists $CacheRef->{'1'}) {
        my $rResultSet = _CallDriver('GetAllInfoUnitOfMeasurement');
        # make copy to primary key cache
        my (@PrimaryKeySet);
        foreach my $hRow (@$rResultSet) {
            my %hPrimaryKey;
            @hPrimaryKey{qw (UnitID)} = @$hRow{qw (UnitID)};
            push @PrimaryKeySet, \%hPrimaryKey;
        }
        $CacheRef->{'1'} = \@PrimaryKeySet;
        # make copy to info cache
        my $InfoCacheRef = GetCache($dbi->cacheKey(),'InfoUnitOfMeasurement');
        if (_DoFillInfoCache('InfoUnitOfMeasurement', scalar @$rResultSet)) {
            foreach my $Row (@$rResultSet) {
                my $InfoCacheString =  join('<||>', @$Row{qw (UnitID)});
                $InfoCacheRef->{$InfoCacheString} = { %$Row };
            }
        }
        return $rResultSet;
    } elsif (_DoFillInfoCache('InfoUnitOfMeasurement', @{$CacheRef->{'1'}})) {
        my @ResultSet;
        foreach my $PrimaryKey (@{$CacheRef->{'1'}}) {
            my $Info = DAL_GetInfoUnitOfMeasurement($PrimaryKey->{'UnitID'});
            push @ResultSet, $Info if defined $Info;
        }
        return \@ResultSet;
    }
    delete $CacheRef->{'1'};
    return DAL_GetAllInfoUnitOfMeasurement();
}

#========================================================================================
# function     DAL_GetInfoUnitOfMeasurement
#----------------------------------------------------------------------------------------
# syntax       $hResult = DAL_GetInfoUnitOfMeasurement($UnitID);
#----------------------------------------------------------------------------------------
# description  returns one dataset
#----------------------------------------------------------------------------------------
# input        $UnitID | object id (primary key) | int
# return       $hResult | with following keys:
#               <ul>
#                   <li>UnitID       - unit id           (primary key)- int
#                   <li>Type         - type of measurement            - string
#                   <li>Factor       - conversion factor to base unit - decimal
#                   <li>Abbreviation - representation symbol          - char(50)
#                   <li>Code         - UN/ECE recommendation 20 code  - char(3)
#               </ul>| reference to hash
#========================================================================================
sub DAL_GetInfoUnitOfMeasurement ($) {
    my ($UnitID) = @_;
    return _CallDriver('GetInfoUnitOfMeasurement', $UnitID) unless $DO_CACHE;

    my $dbi = GetCurrentDBHandle();
    my $CacheRef    = GetCache($dbi->cacheKey(),'InfoUnitOfMeasurement');
    my $CacheString =  $UnitID;
    $CacheRef->{$CacheString} = _CallDriver('GetInfoUnitOfMeasurement', $UnitID) unless exists $CacheRef->{$CacheString};
    return undef unless defined $CacheRef->{$CacheString};
    return {%{$CacheRef->{$CacheString}}};
}

#========================================================================================
# function     DAL_GetPKeyUnitOfMeasurementByCode
#----------------------------------------------------------------------------------------
# syntax       $rahResult = DAL_GetPKeyUnitOfMeasurementByCode($Code);
#----------------------------------------------------------------------------------------
# description  get primary keys of table unitofmeasurement by code
#----------------------------------------------------------------------------------------
# input        $Code | code id | string
# return       $rahResult | with following keys:
#               <ul>
#                   <li>UnitID - unit id (primary key)- int
#               </ul>|reference to array of hashes
#========================================================================================
sub DAL_GetPKeyUnitOfMeasurementByCode ($) {
    my ($Code) = @_;
    return _CallDriver('GetPKeyUnitOfMeasurementByCode', $Code) unless $DO_CACHE;

    my $dbi = GetCurrentDBHandle();
    my $CacheRef = GetCache($dbi->cacheKey(),'UnitOfMeasurementByCode');
    my $CacheString =  $Code;
    $CacheRef->{$CacheString} = _CallDriver('GetPKeyUnitOfMeasurementByCode', $Code) unless exists $CacheRef->{$CacheString};
    return [ @{$CacheRef->{$CacheString}} ];
}

#========================================================================================
# function     DAL_GetInfoUnitOfMeasurementByCode
#----------------------------------------------------------------------------------------
# syntax       $rahResult = DAL_GetInfoUnitOfMeasurementByCode($Code);
#----------------------------------------------------------------------------------------
# description  get full row of table unitofmeasurement by code
#----------------------------------------------------------------------------------------
# input        $Code | code id | string
# return       $rahResult | with following keys:
#               <ul>
#                   <li>UnitID       - unit id           (primary key)- int
#                   <li>Type         - type of measurement            - string
#                   <li>Factor       - conversion factor to base unit - decimal
#                   <li>Abbreviation - representation symbol          - char(50)
#                   <li>Code         - UN/ECE recommendation 20 code  - char(3)
#               </ul>|reference to array of hashes
#========================================================================================
sub DAL_GetInfoUnitOfMeasurementByCode ($);
sub DAL_GetInfoUnitOfMeasurementByCode ($) {
    my ($Code) = @_;
    return _CallDriver('GetInfoUnitOfMeasurementByCode', $Code) unless $DO_CACHE;

    my $dbi = GetCurrentDBHandle();
    my $CacheRef = GetCache($dbi->cacheKey(),'UnitOfMeasurementByCode');
    my $CacheString =  $Code;
    my $CacheContent = $CacheRef->{$CacheString};
    if (not defined $CacheContent) {
        my $rResultSet = _CallDriver('GetInfoUnitOfMeasurementByCode', $Code);
        my (@PrimaryKeySet);
        foreach my $hRow (@$rResultSet) {
            my %hPrimaryKey;
            @hPrimaryKey{qw (UnitID)} = @$hRow{qw (UnitID)};
            push @PrimaryKeySet, \%hPrimaryKey;
        }
        $CacheRef->{$CacheString} = \@PrimaryKeySet;
        my $InfoCacheRef = GetCache($dbi->cacheKey(),'InfoUnitOfMeasurement');
        if (_DoFillInfoCache('InfoUnitOfMeasurement', scalar @$rResultSet)) {
            foreach my $Row (@$rResultSet) {
                my $InfoCacheString =  @$Row{qw (UnitID)};
                $InfoCacheRef->{$InfoCacheString} = { %$Row };
            }
        }
        return $rResultSet;
    } elsif (_DoFillInfoCache('InfoUnitOfMeasurement', scalar @$CacheContent)) {
        my @ResultSet;
        foreach my $PrimaryKey (@$CacheContent) {
            my $Info = DAL_GetInfoUnitOfMeasurement($PrimaryKey->{'UnitID'});
            push @ResultSet, $Info if defined $Info;
        }
        return \@ResultSet;
    }
    delete $CacheRef->{$CacheString};
    return DAL_GetInfoUnitOfMeasurementByCode($Code);
}

#========================================================================================
# function     DAL_GetPKeyUnitOfMeasurementByAbbreviation
#----------------------------------------------------------------------------------------
# syntax       $rahResult = DAL_GetPKeyUnitOfMeasurementByAbbreviation($Abbreviation);
#----------------------------------------------------------------------------------------
# description  get primary keys of table unitofmeasurement by abbreviation
#----------------------------------------------------------------------------------------
# input        $Abbreviation | abbreviation id | string
# return       $rahResult | with following keys:
#               <ul>
#                   <li>UnitID - unit id (primary key)- int
#               </ul>|reference to array of hashes
#========================================================================================
sub DAL_GetPKeyUnitOfMeasurementByAbbreviation ($) {
    my ($Abbreviation) = @_;
    return _CallDriver('GetPKeyUnitOfMeasurementByAbbreviation', $Abbreviation) unless $DO_CACHE;

    my $dbi = GetCurrentDBHandle();
    my $CacheRef = GetCache($dbi->cacheKey(),'UnitOfMeasurementByAbbreviation');
    my $CacheString =  $Abbreviation;
    $CacheRef->{$CacheString} = _CallDriver('GetPKeyUnitOfMeasurementByAbbreviation', $Abbreviation) unless exists $CacheRef->{$CacheString};
    return [ @{$CacheRef->{$CacheString}} ];
}

#========================================================================================
# function     DAL_GetInfoUnitOfMeasurementByAbbreviation
#----------------------------------------------------------------------------------------
# syntax       $rahResult = DAL_GetInfoUnitOfMeasurementByAbbreviation($Abbreviation);
#----------------------------------------------------------------------------------------
# description  get full row of table unitofmeasurement by abbreviation
#----------------------------------------------------------------------------------------
# input        $Abbreviation | code id | string
# return       $rahResult | with following keys:
#               <ul>
#                   <li>UnitID       - unit id           (primary key)- int
#                   <li>Type         - type of measurement            - string
#                   <li>Factor       - conversion factor to base unit - decimal
#                   <li>Abbreviation - representation symbol          - char(50)
#                   <li>Code         - UN/ECE recommendation 20 code  - char(3)
#               </ul>|reference to array of hashes
#========================================================================================
sub DAL_GetInfoUnitOfMeasurementByAbbreviation ($);
sub DAL_GetInfoUnitOfMeasurementByAbbreviation ($) {
    my ($Abbreviation) = @_;
    return _CallDriver('GetInfoUnitOfMeasurementByAbbreviation', $Abbreviation) unless $DO_CACHE;

    my $dbi = GetCurrentDBHandle();
    my $CacheRef = GetCache($dbi->cacheKey(),'UnitOfMeasurementByAbbreviation');
    my $CacheString =  $Abbreviation;
    my $CacheContent = $CacheRef->{$CacheString};
    if (not defined $CacheContent) {
        my $rResultSet = _CallDriver('GetInfoUnitOfMeasurementByAbbreviation', $Abbreviation);
        my (@PrimaryKeySet);
        foreach my $hRow (@$rResultSet) {
            my %hPrimaryKey;
            @hPrimaryKey{qw (UnitID)} = @$hRow{qw (UnitID)};
            push @PrimaryKeySet, \%hPrimaryKey;
        }
        $CacheRef->{$CacheString} = \@PrimaryKeySet;
        my $InfoCacheRef = GetCache($dbi->cacheKey(),'InfoUnitOfMeasurement');
        if (_DoFillInfoCache('InfoUnitOfMeasurement', scalar @$rResultSet)) {
            foreach my $Row (@$rResultSet) {
                my $InfoCacheString =  @$Row{qw (UnitID)};
                $InfoCacheRef->{$InfoCacheString} = { %$Row };
            }
        }
        return $rResultSet;
    } elsif (_DoFillInfoCache('InfoUnitOfMeasurement', scalar @$CacheContent)) {
        my @ResultSet;
        foreach my $PrimaryKey (@$CacheContent) {
            my $Info = DAL_GetInfoUnitOfMeasurement($PrimaryKey->{'UnitID'});
            push @ResultSet, $Info if defined $Info;
        }
        return \@ResultSet;
    }
    delete $CacheRef->{$CacheString};
    return DAL_GetInfoUnitOfMeasurementByAbbreviation($Abbreviation);
}

1;
