#========================================================================================
# package      DE_EPAGES::Presentation::API::Cartridge
# base         DE_EPAGES::Presentation::API::PresentationInstaller
# state        public
#----------------------------------------------------------------------------------------
# description  This is the main cartridge class for install/patch/uninstall.
#========================================================================================
package DE_EPAGES::Presentation::API::Cartridge;
use base DE_EPAGES::Presentation::API::PresentationInstaller;

use strict;
use DE_EPAGES::Object::API::Factory qw( ExistsClassByAlias LoadClassByAlias);
use DE_EPAGES::Trigger::API::Table::Hook qw (ExistsHookByName GetPKeyHookByName InsertHook);
use DE_EPAGES::Trigger::API::Table::HookFunction qw (
    DeleteHookFunction
    ExistsHookFunctionByHookID
    GetPKeyHookFunctionByHookID
);

#========================================================================================
# function     new
# state        private
#----------------------------------------------------------------------------------------
# syntax       $Installer = DE_EPAGES::Presentation::API::Cartridge->new(%options);
#----------------------------------------------------------------------------------------
# description  Creates a new installer object of DE_EPAGES::Presentation.
#----------------------------------------------------------------------------------------
# input        %options | options for un/install
#               <ul>
#                 <li>IsRecursive - recursive un/installation of required or dependent
#                     cartridges
#               </ul>
#               | hash
# return       $Installer | cartridge installer | object
#========================================================================================
sub new {
    my ($class, %options) = @_;

    my $self = __PACKAGE__->SUPER::new(
        %options,
        'CartridgeDirectory' => 'DE_EPAGES/Presentation',
        'Version'            => '7.42.0',
        'Patches'            => ['6.15.1','6.15.2','6.15.3','6.15.4','6.16.0','6.16.1','6.16.2','6.16.3','6.17.0','6.17.1','6.17.2','6.17.3','6.17.4','6.17.5','6.17.6','6.17.7','6.17.8','6.17.9','6.17.9.1','6.17.9.2','6.17.9.3','6.17.10','6.17.11','6.17.12','6.17.12.1','6.17.12.2','6.17.13','6.17.13.1','6.17.13.2','6.17.14','6.17.14.1','6.17.15','6.17.15.1','6.17.16','6.17.17','6.17.17.1','6.17.18','6.17.19','6.17.20','6.17.20.1','6.17.21','6.17.21.1','6.17.22','6.17.23','6.17.24','6.17.24.1','6.17.25','6.17.26','6.17.26.1','6.17.27','6.17.27.1','6.17.28','6.17.28.1','6.17.29','6.17.29.1','6.17.30','6.17.31','6.17.32','6.17.33','6.17.33.1','6.17.34','6.17.34.1','6.17.35','6.17.36','6.17.36.1','6.17.37','6.17.38','6.17.38.1','6.17.39','6.17.39.1','6.17.40','6.17.40.1','6.17.41','6.17.41.1','6.17.42','6.17.43','6.17.43.1','6.17.44','6.17.45','6.17.45.1','6.17.46','6.17.47','6.17.47.1','6.17.48','6.17.48.1','6.17.49','6.17.49.1','6.17.50','6.17.51','6.17.51.1','6.17.52','6.17.53','7.0.0','7.1.0','7.2.0','7.3.0','7.4.0','7.5.0','7.6.0','7.7.0','7.8.0','7.9.0','7.10.0','7.11.0','7.12.0','7.13.0','7.14.0','7.15.0','7.16.0','7.17.0','7.18.0','7.19.0','7.20.0','7.21.0','7.22.0','7.23.0','7.24.0','7.25.0','7.26.0','7.27.0','7.28.0','7.29.0','7.30.0','7.31.0','7.32.0','7.33.0','7.34.0','7.35.0','7.36.0','7.37.0','7.38.0','7.39.0','7.40.0','7.41.0'],
    );
    return bless $self, $class;
}

#========================================================================================
# function     install
# state        private
#----------------------------------------------------------------------------------------
# syntax       $Installer->install;
#----------------------------------------------------------------------------------------
# description  Installs the cartridge.
#========================================================================================
sub install {
    my $self = shift;

    $self->startInstall;
    $self->SUPER::install;
    $self->finishInstall;
    return;
}

#========================================================================================
# function     uninstall
# state        private
#----------------------------------------------------------------------------------------
# syntax       $Installer->uninstall;
#----------------------------------------------------------------------------------------
# description  Uninstalls the cartridge.
#========================================================================================
sub uninstall {
    my $self = shift;

    $self->startUninstall;
    $self->SUPER::uninstall;
    $self->finishUninstall;
    return;
}

sub doPatch {
    my $self = shift;

    # TODO: remove this code when the the patch does no longer support 6.0.7 or early versions
    if( ExistsHookByName('OBJ_AfterUpdateMenu') ) {
        # EPG-13763: avoid error: Undefined subroutine &DE_EPAGES::Shop::Hooks::Menu::OnInsertMenu
        my $HookID = GetPKeyHookByName('OBJ_AfterUpdateMenu');
        if( ExistsHookFunctionByHookID($HookID, 'DE_EPAGES::Shop::Hooks::Menu::OnInsertMenu') ) {
            my $HookFunctionID = GetPKeyHookFunctionByHookID($HookID, 'DE_EPAGES::Shop::Hooks::Menu::OnInsertMenu');
            DeleteHookFunction($HookFunctionID);
        }
    }
    else {
        # avoid warning in patch: Hook doesn't exist (Name=OBJ_AfterUpdateMenu)
        InsertHook({ 'Name' => 'OBJ_AfterUpdateMenu' });
    }

    return $self->SUPER::doPatch;
}

#========================================================================================
# function     patchDBStructure_6_15_2
# state        private
#----------------------------------------------------------------------------------------
# syntax       $Installer->patchDBStructure_6_15_2;
#----------------------------------------------------------------------------------------
# description  Patches cartridge db structure in active store (database connection)
#               from version '6.15.2' to version '6.15.3'.
#----------------------------------------------------------------------------------------
# return       $Version | version of cartridge after patch function, '6.15.3' | string
#========================================================================================
sub patchDBStructure_6_15_2 {
    my $self = shift;

    my $dbi = $self->getCurrentDBHandle();

    if($dbi->driverName() eq 'mysql'){
        #changes for file /Database/mysql/Tables/template.sql
        $self->logInfo('drop indices \'i_template_pagetypeid\' from table \'template\'');
        $dbi->dropIndexesIfExist('template', ['i_template_pagetypeid']);
    }

    return '6.15.3';
}

#========================================================================================
# function     patchDBStructure_6_17_4
# state        private
#----------------------------------------------------------------------------------------
# syntax       $Installer->patchDBStructure_6_17_4;
#----------------------------------------------------------------------------------------
# description  Patches cartridge db structure in active store (database connection)
#               from version '6.17.4' to version '6.17.5'.
#----------------------------------------------------------------------------------------
# return       $Version | version of cartridge after patch function, '6.17.5' | string
#========================================================================================
sub patchDBStructure_6_17_4 {
    my $self = shift;
    #dont use API function, to avoid inconistencies between perl modules and database content

    my $dbi = $self->getCurrentDBHandle();

    if($dbi->driverName() eq 'mysql'){

        #changes for file /Database/mysql/Tables/session.sql
        $self->logInfo('change type of column session.token');
        $dbi->modifyTableColumn('session', 'token varchar(32) not null');

        unless ($dbi->existsTableColumn('session', 'lastactivity')) {
            $self->logInfo('add column session.lastactivity after column session.token');
            $dbi->addTableColumnAfterColumn('session', 'token', 'lastactivity datetime not null default 0');
            $dbi->modifyTableColumnDropDefault('session','lastactivity');
        }

    }

    return '6.17.5';
}

#========================================================================================
# function     patchDBStructure_6_17_5
# state        private
#----------------------------------------------------------------------------------------
# syntax       $Installer->patchDBStructure_6_17_5;
#----------------------------------------------------------------------------------------
# description  Patches cartridge db structure in active store (database connection)
#               from version '6.17.5' to version '6.17.6'.
#----------------------------------------------------------------------------------------
# return       $Version | version of cartridge after patch function, '6.15.3' | string
#========================================================================================
sub patchDBStructure_6_17_5 {
    my $self = shift;

    my $dbi = $self->getCurrentDBHandle();

    if($dbi->driverName() eq 'mysql'){
        #changes for file /Database/mysql/Tables/template.sql
        $self->logInfo('fill proper value for the new column "lastactivity" for old sessions');
        $dbi->do(<<END_SQL);
UPDATE session
SET lastactivity=creationdate
WHERE lastactivity=0
END_SQL
    }

    $self->importPatchDir('Patch_6_17_5', 'PatchDBStructure');

    return '6.17.6';
}

#========================================================================================
# function     patchDBStructure_6_17_8
# state        private
#----------------------------------------------------------------------------------------
# syntax       $Installer->patchDBStructure_6_17_8;
#----------------------------------------------------------------------------------------
# description  Patches cartridge db structure in active store (database connection)
#               from version '6.17.8' to version '6.17.9'.
#----------------------------------------------------------------------------------------
# return       $Version | version of cartridge after patch function, '6.17.9' | string
#========================================================================================
sub patchDBStructure_6_17_8 {
    my $self = shift;
    #dont use API function, to avoid inconistencies between perl modules and database content

    my $dbi = $self->getCurrentDBHandle();

    if($dbi->driverName() eq 'mysql'){
        if( ExistsClassByAlias('HelpTopic') ) {
            my $ClassID = $self->getClassID('HelpTopic');

            $self->logInfo('delete table pagetypehelptopic');
            $dbi->do(<<'END_SQL');
DROP TABLE pagetypehelptopicext
END_SQL

            $self->logInfo('delete table pagetypehelptopicext');
            $dbi->do(<<'END_SQL');
DROP TABLE pagetypehelptopic
END_SQL

            $self->logInfo('delete table helptopic');
            $dbi->do(<<'END_SQL');
DROP TABLE helptopic
END_SQL

            # delete stringattribute first
            $self->logInfo('delete localizedstringattributes first');
            $dbi->do(<<'END_SQL',[$ClassID]);
DELETE la FROM localizedstringattribute la
INNER JOIN attribute a ON a.attributeid = la.attributeid
INNER JOIN class c ON c.classid = a.attributeclassid
WHERE c.classid = ?
END_SQL

            # set package and delete class
            $self->logInfo('set package and delete class');
            $dbi->do(<<'END_SQL',[$ClassID]);
UPDATE class SET package = 'DE_EPAGES::Object::API::Object::Object' WHERE classid = ?
END_SQL

            LoadClassByAlias('HelpTopic')->delete;
        }
    }

    $self->importPatchDir('Patch_6_17_8', 'PatchDBStructure');

    return '6.17.9';
}

#========================================================================================
# function     patchCartridgeDir_7_12_0
# state        private
#----------------------------------------------------------------------------------------
# syntax       $NewVersion = $Installer->patchCartridgeDir_7_12_0;
#----------------------------------------------------------------------------------------
# description  Patches Cartridge directory
#               from version '7.12.0' to version '7.13.0'.
#----------------------------------------------------------------------------------------
# return       $NewVersion | new version | string
#========================================================================================
sub patchCartridgeDir_7_12_0 {
    my $self = shift;

    $self->onPatchDeleteCartridgeDirectory('bin');

    return '7.13.0';
}

1;
