#========================================================================================
# package      DE_EPAGES::PriceList::Hooks::ProductService
# state        public
# description  extends PoductService with elements
#========================================================================================
package DE_EPAGES::PriceList::Hooks::ProductService;

use strict;
use DE_EPAGES::PriceList::API::Constants qw( UNITY_PRICELIST_PREFIX );

#========================================================================================
# function     OnProductServiceGetProductData
# state        public
#----------------------------------------------------------------------------------------
# syntax       OnProductServiceGetProductData( $hParams );
#----------------------------------------------------------------------------------------
# description  Hook function for the API_ProductService_GetProductData hook.
#               Extends the ProductService result (hResult) with bulk price informations
#               for the given product.
#----------------------------------------------------------------------------------------
# input        $hParams |<ul>
#                   <li>Product - product - object</li>
#                   <li>hResult - ProductService result - ref.hash</li>
#                   <li>ProductInfo - some usefull product attribute values from product - ref.hash</li>
#                   <li>ProductData - additional elements from shop needed for formatting etc.  - ref.hash</li>
#                   </ul> | ref.hash
#========================================================================================
sub OnProductServiceGetProductData {
my ($hParams) = @_;

    my $Product               = $hParams->{'Product'};
    my $hProductServiceResult = $hParams->{'hResult'};
    my $hProductInfo          = $hParams->{'ProductInfo'};
    my $hProductData          = $hParams->{'ProductData'};
    my $Shop                  = $Product->getSite();
    my $CurrencyID            = $hProductData->{'CurrencyID'};

    # do this only for unity
    return unless $Shop->get('IsUnityRegistered');

    # get bulk prices for product with the predfined pricelist alias
    my $PriceListAlias = sprintf('%s%s', UNITY_PRICELIST_PREFIX, $CurrencyID);

    return unless $Shop->folder('PriceLists')->existsChild($PriceListAlias);
    my $PriceList = $Shop->folder('PriceLists')->child($PriceListAlias);
    my $aPrices = $PriceList->getScalePricesByProductID($Product->id, $Shop->get('TaxModel'), $Product->getWithParent('TaxClass'));

    return unless scalar @$aPrices;

    my $Formatter  = $hProductData->{'Formatter'};
    my $TaxModel   = $hProductData->{'TaxModel'};

    #build structure for bulk prices and add values
    my $BulkPrices = [];
    foreach my $hBulkPrice (@$aPrices){
        my $hPriceInfo = $Formatter->price( $hBulkPrice->{'Price'}, $CurrencyID, $TaxModel );
        my $hQuantity = $Formatter->quantity( $hBulkPrice->{'Quantity'}, $hProductInfo->{'OrderUnit'} );
        push @$BulkPrices, {
                            'price'    => $hPriceInfo,
                            'quantity' => $hQuantity
                           };
    }
    push @{$hProductServiceResult->{'bulkPriceInfo'}}, @$BulkPrices ;
}

1;
