#==============================================================================
# package      DE_EPAGES::Shipping::UI::Shop
# state        private
#==============================================================================
package DE_EPAGES::Shipping::UI::Shop;
use base DE_EPAGES::Presentation::UI::Object;

use strict;

use DE_EPAGES::Core::API::Error qw ( Error ExistsError GetError ResetError );
use DE_EPAGES::Core::API::PerlTools qw ( isValid );
use DE_EPAGES::Object::API::Factory qw ( LoadRootObject Transaction );
use DE_EPAGES::Shipping::API::Shop qw ( GetShippingMethodsFolder );
use DE_EPAGES::Shipping::API::Tools qw (
    SetLinkShippingProviderMethod
    SetLogoToShippingMethod
);
use DE_EPAGES::Trigger::API::Trigger qw ( TriggerHook );

#==============================================================================
# Event handlers
#==============================================================================
sub setDefault {
    my $self = shift;
    my ($ShippingMethod) = @_;

    # all shipping methods
    my $ShopMethods = $ShippingMethod->getSite()->get('ShippingMethods');

    # if only one then this is default
    if (scalar @$ShopMethods == 1) {
        $ShippingMethod->set({'IsDefault'=>1});
        return;
    }

    # all defaults without current
    my @DefaultMethods = grep { $_->get('IsDefault') and $ShippingMethod->id != $_->id } @$ShopMethods;

    # current default
    my $IsDefault = $ShippingMethod->get('IsDefault');

    if ($IsDefault) {
        # set all non current to non default
        $_->set({'IsDefault'=>0}) foreach @DefaultMethods;
    } elsif (scalar @DefaultMethods > 1) {
        # multible defaults to non default
        shift @DefaultMethods;
        $_->set({'IsDefault'=>0}) foreach @DefaultMethods;
    } else { # scalar @DefaultMethods == 0
        # set first not current to default
        my @NewDefaultMethods = grep { $ShippingMethod->id != $_->id } @$ShopMethods;
        $NewDefaultMethods[0]->set({'IsDefault'=>1});
    }
    return;
}

sub UpdateShippingMethods {
    my $self = shift;
    my ($Servlet) = @_;

    my $Shop = $Servlet->object;
    my $DefaultMethodID = $Servlet->params('IsDefault');
    my $ahListItems = $self->listItems( $Servlet, $Shop, 'ShippingMethod' );

    # test if default is visible
    if (defined $DefaultMethodID) {
        my $IsDefaultVisible = $Shop->loadObject($DefaultMethodID, 'ShippingMethod')->get('IsActivated');
        return $Servlet->vars('INVISIBLE_DEFAULT',1) unless $IsDefaultVisible;
    }

    # save default and position
    foreach my $hListItem (@$ahListItems) {
        my $Method = $hListItem->{'Object'};
        $Method->set({
            'IsDefault' => $DefaultMethodID == $Method->id ? 1 : 0,
            'Position' => $hListItem->{'Position'},
        });
    }
    GetShippingMethodsFolder($Shop)->renumberChildren;

    return;

}

sub SetShippingMethodVisible {
    my $self = shift;
    my ($Servlet) = @_;

    my $Shop = $Servlet->object;
    my @ShippingMethodIDs = $Servlet->form->value('ListObjectID');
    my $aShippingMethods  = $Shop->loadObjects( \@ShippingMethodIDs, 'ShippingMethod' );
    my $aShopCurrencies = $Shop->get('Currencies');
    my @ActiveCurrencies = grep { $_->{'IsActivated'} == 1 } @$aShopCurrencies;
    my $hVars = { 'TaxModel'   => $Shop->get('TaxModel') };
    foreach my $ShippingMethod (@{$aShippingMethods}) {
        my $ShippingObjectMethod = $self->_getSelectedShippingObjectMethod($ShippingMethod);
        my $FulfillmentUsable = defined($ShippingObjectMethod) ? $ShippingObjectMethod->isUsable() : 1;
        my $ShippingUsable = 1;
        foreach my $ActiveCurrency (@ActiveCurrencies) {
            $hVars->{'CurrencyID'} = $ActiveCurrency->{'CurrencyID'};
            my $SettingsComplete = $ShippingMethod->usableAtStorefront($hVars);
            if ($SettingsComplete == 0 || $FulfillmentUsable == 0) {
                $ShippingUsable = $SettingsComplete;
                $Servlet->form->executeFormError({
                    'Reason'=> 'SETTINGS_INCOMPLETE',
                    'Name'  => 'IsActivated',
                    'Value' => $ShippingMethod->alias,
                });
            }
        }

        $ShippingMethod->set({ 'IsActivated' => ($ShippingUsable && $FulfillmentUsable)});
    }

    return;
}

sub SetShippingMethodNotVisible {
    my $self = shift;
    my ($Servlet) = @_;

    my $IsDefault = $Servlet->params('IsDefault');
    if (not defined $IsDefault) {
        $Servlet->vars('INVISIBLE_DEFAULT', 1);
        return;
    }

    my $Shop = $Servlet->object;
    my @ShippingMethodIDs = $Servlet->form->value('ListObjectID');
    my $aShippingMethods = $Shop->loadObjects( \@ShippingMethodIDs, 'ShippingMethod' );
    my $DefaultShippingMethod = $Shop->loadObject($IsDefault, 'ShippingMethod');
    foreach my $ShippingMethod (@{$aShippingMethods}) {
        $DefaultShippingMethod->id == $ShippingMethod->id
            ? $Servlet->vars('INVISIBLE_DEFAULT', 1)
            : $ShippingMethod->set({ 'IsActivated' => 0 });
    }
    return;
}


sub DeleteShippingMethods {
    my $self = shift ;
    my ($Servlet) = @_ ;

    my $Shop = $Servlet->object ;

    $self->SUPER::DeleteListedObjects($Servlet) ;

    $Shop->child('ShippingMethods')->renumberChildren ;

    # Clean the pages cached to generate the MBO main menu
    $Shop->clearSavedPages() ;

    return;
}

#==================================================================================================
# function     AddNewShippingMethods
# state        public
#--------------------------------------------------------------------------------------------------
# syntax       AddNewShippingMethods( $Servlet ) ;
#--------------------------------------------------------------------------------------------------
# description  Method activated by the ChangeAction 'AddNewShippingMethod' which is called from
#               the ViewAction 'MBO-ViewShippingList' to create a new ShippingMethod.
#--------------------------------------------------------------------------------------------------
# input        $Servlet
#                   | The <b>Servlet</b> object for the request
#                   | object
#==================================================================================================
sub AddNewShippingMethods {
    my $self = shift ;
    my ( $Servlet ) = @_ ;

    my $Shop = $Servlet->object;

    # Input data ( form ) validation
    $Servlet->form->form( $Shop, 'NewShippingMethod' );

    $self->_processFormData( $Servlet ) ;

    my $NewShippingMethodObject = undef ;
    my $ShippingFulfillmentObject = undef ;

    Transaction( sub {
        ( $NewShippingMethodObject , $ShippingFulfillmentObject ) =
            $self->_createShippingObjects( $Servlet ) ;
    });

    # We assign the logistic logo to the new object

    SetLogoToShippingMethod(
        $NewShippingMethodObject,
        $ShippingFulfillmentObject
    ) ;

    # Finally we have to return some values to the Form related to the finished operation
    # so it could decide what to do ...

    # IMPORTANT : The ID of the new Shipping Method created in the operation

    $Servlet->vars(
        'NewShippingMethodID',
        $NewShippingMethodObject->id
    ) ;

    # And also important the Settings TAB which to jump when close the dialog ... because the
    # Free delivery method without a logistic hasn't a specific Settings TAB we need to jump to
    # the General
    $Servlet->vars('GotoView', $NewShippingMethodObject->hasSettings ? 'MBO-ViewSettings' : 'MBO-ViewGeneral' );

    # Clean the pages cached to generate the MBO main menu
    $Shop->clearSavedPages() ;

    return ;
}
#   END    AddNewShippingMethods
#==================================================================================================

#==================================================================================================
# function     _createShippingObjects
# state        private
#--------------------------------------------------------------------------------------------------
# syntax       _createShippingObjects( $Servlet ) ;
#--------------------------------------------------------------------------------------------------
# description  Method called to carries out the underlying process of:
#
#                   . Creates a new Delivery Method
#                   . If the selected logistic is a type, create his logistic method.
#                   . Links ( dependencies ) the new Delivery Method with the logistic method.
#
#--------------------------------------------------------------------------------------------------
# input        $Servlet
#                   | The <b>Servlet</b> object for the request
#                   | object
# return       $NewShippingMethodObject
#                   | ShippingMethod
#                   | object
# return       $ShippingFulfillmentObject
#                   | ShippingFulfillmentMethod
#                   | object
#==================================================================================================
sub _createShippingObjects {
    my $self = shift ;
    my ( $Servlet ) = @_ ;

    my $ShippingFulfillmentObject = undef ;

    my $NewShippingMethodObject = $self->_newShippingMethod( $Servlet ) ;

    my $ShippingFulfillmentTypeID =
        $Servlet->params->{'NewShippingFulfillmentTypeID'} ;

    # If the merchant has selected the 'User Defined' shipping method ( ID forced to 0 )
    # we don't have to do anything else ...

    if ( $ShippingFulfillmentTypeID != 0 ) {

        # The merchant has select a Shipping Fulfillment !!!

        # We have to see if the selected item is a ShippingFulfillmentType or a previously
        # created ShippingFulfillmentMethod (?)

        my $ShippingFulfillmentMethodID =
            $Servlet->params->{'ShippingFulfillmentMethodID'} ;

        # OK ... Is a ShippingFulfillmentMethod ?

        if ( isValid( $ShippingFulfillmentMethodID ) ) {
            my $ShippingFulfillmentType = LoadRootObject()->loadObject(
                $Servlet->params->{'NewShippingFulfillmentTypeID'},
                'ShippingFulfillmentType'
            ) ;

            my $ShippingProviderTypes = $ShippingFulfillmentType->get('ShippingProviderTypes');

            if (scalar @$ShippingProviderTypes){
                $ShippingFulfillmentObject = $self->_getShippingProviderMethod(
                    $Servlet,
                    $ShippingFulfillmentMethodID,
                    $ShippingFulfillmentType
                ) ;
                SetLinkShippingProviderMethod($NewShippingMethodObject,$ShippingFulfillmentObject);
            } else {
                $ShippingFulfillmentObject = $self->_getShippingFulfillmentMethod(
                    $Servlet,
                    $ShippingFulfillmentMethodID,
                    $ShippingFulfillmentType
                ) ;
                $self->_setLinkShippingFulfillmentMethod($NewShippingMethodObject,$ShippingFulfillmentObject);
            }

        } else {

            my $ShippingFulfillmentType = LoadRootObject()->loadObject(
                $Servlet->params->{'NewShippingFulfillmentTypeID'},
                'ShippingFulfillmentType'
                ) ;

            my $ShippingProviderTypes = $ShippingFulfillmentType->get('ShippingProviderTypes');

            if (scalar @$ShippingProviderTypes){
                $ShippingFulfillmentObject = $self->_newShippingProviderMethod( $Servlet ) ;
                SetLinkShippingProviderMethod($NewShippingMethodObject,$ShippingFulfillmentObject);
            } else {
                $ShippingFulfillmentObject = $self->_newShippingFulfillmentMethod( $Servlet ) ;
                $self->_setLinkShippingFulfillmentMethod($NewShippingMethodObject,$ShippingFulfillmentObject);
            }
        }
    }

    return ( $NewShippingMethodObject , $ShippingFulfillmentObject ) ;
}
#   END    AddNewShippingMethods
#==================================================================================================

#==================================================================================================
# function     _newShippingMethod
# state        private
#--------------------------------------------------------------------------------------------------
# syntax       _newShippingMethod( $Servlet ) ;
#--------------------------------------------------------------------------------------------------
# description Creates a new ShippingMethod
#--------------------------------------------------------------------------------------------------
# input        $Servlet
#                   | The Servlet object for the request
#                   | object
# return       $NewShippingMethodObject
#                   | The new ShippingMethod created
#                   | object
#==================================================================================================
sub _newShippingMethod {
    my $self = shift;

    my ($Servlet) = @_;

    my $Shop = $Servlet->object;

    my $Form = $Servlet->form->form($Shop, 'NewShippingMethod');

    my $ShippingType = LoadRootObject()->loadObject($Form->{'NewShippingTypeID'}, 'ShippingType');
    delete $Form->{'NewShippingTypeID'};

    $self->_checkFeatureAvailable( $Servlet, $ShippingType->alias, 'NewShippingTypeID' ) ;

    my %Form;
    $Form{$_}= $Form->{'New'.$_} foreach map { s/^New// && $_} grep { /^New/ } keys %$Form;
    $Form{'Shop'} = $Shop;

    # test if Method with the given name exists
    my $Folder = GetShippingMethodsFolder($Shop);


    if($Shop->get('IsUnityRegistered')){
        my $NewAlias = $Form{'Alias'};
        my $TestAlias = $NewAlias;
        my $Counter = 1;

        while ($Folder->existsChild($TestAlias)) {
            $TestAlias = $NewAlias . '-' . $Counter;
            $Counter++;
        }
        $Form{'Alias'} = $TestAlias;
        $Form{'Name'} =  $NewAlias;
    } else {
        if ($Form{'Alias'} && $Folder->existsChild($Form{'Alias'})){
            $Servlet->vars('OBJECT_EXISTS',1);
            $Servlet->form->executeFormError({
                'Reason'=> 'OBJECT_EXISTS',
                'Name'  => 'NewAlias',
                'Value' => $Form{'Alias'},
            }) ;
        }
    }

    my $ShopMethods = $Shop->get('ShippingMethods');
    $Form{'IsDefault'} = 1 unless scalar @$ShopMethods;
    my $TaxMatrix = $Shop->get('TaxMatrix');
    $Form{'TaxClass'} = $TaxMatrix->defaultTaxClass unless defined $Form{'TaxClass'};

    # if featureMaxValue is 1, insert only if accepted if confirmation is used
    if ($Shop->featureMaxValue('ShippingMethods') == 1) {
        if ($Servlet->user->get('DeleteConfirmation') and not $Servlet->params('ACCEPT')) {
            $Servlet->vars('OVERWRITE',1);
            $Servlet->vars('NewAlias', $Form{'NewAlias'} );
            $Servlet->vars('NewShippingTypeID', $ShippingType->id);
            return;
        } else {
            # else reset delete
            $Servlet->vars('OVERWRITE',0);
        }
    }

    my $NewShippingMethodObject ;

    eval {
        $NewShippingMethodObject = $ShippingType->newShippingMethod($Shop, \%Form);
    };

    if (ExistsError()) {
        my $Error = GetError();
        Error($Error) unless($Error->code eq 'FeatureLimitExceeded');
        ResetError();

        $Servlet->vars('FeatureMaxValueReached', $Error->vars);
    }

    $Shop->child('ShippingMethods')->renumberChildren;

    # Update products by default for new shipping methods to avoid new shipping methods not being applied to
    # old products after one deactivated shipping method was created. See UNITY-4724
    my $aShopShippingMethods = $Shop->get('ShippingMethods');

    # go through all products and set the shipping method as defined
    my $ProductFolder = $Shop->get('ProductFolder');
    my $aProductIDs = defined $ProductFolder ? $ProductFolder->findChildren : [];
    foreach my $ProductID ( @$aProductIDs ) {
        my $Product = $Shop->loadObject($ProductID, 'Product');
        # If the product is a master product, also add the subproducts to the array of product ids
        if ($Product->get('IsMaster') == 1) {
            push(@$aProductIDs, map {$_->id} @{$Product->get('SubProducts')});
        }

        # Get all shipping methods
        my $aProductShippingMethods = $Product->get('ShippingMethods');

        if ( scalar(@$aProductShippingMethods) ) {
            push(@$aProductShippingMethods, $NewShippingMethodObject);
        }


        #if all shipping methods are enabled then this attribute should be undef
        if (scalar(@$aShopShippingMethods) <= scalar(@$aProductShippingMethods)) {
            $Product->set({'ShippingMethods' => undef});
        }
        else {
            $Product->set({'ShippingMethods' => $aProductShippingMethods});
        }
    }

    return $NewShippingMethodObject;
}

#==================================================================================================
# function     _processFormData
# state        private
#--------------------------------------------------------------------------------------------------
# syntax       _processFormData( $Servlet ) ;
#--------------------------------------------------------------------------------------------------
# description  Checks the format of the input Form field 'NewShippingFulfillmentTypeID', which
#               could have the formats:
#
#                   <ID_A>              One ID
#                   <ID_A>-<ID_B>       Two ID's separated by the - char
#
#               where
#
#                   ID_A                ShippingFulfillmentTypeID
#                   ID_B                ShippingFulfillmentMethodID
#
#               The parsed fields are inserted in:
#
#                   ID_A    =>          $Servlet->params->{'NewShippingFulfillmentTypeID'}
#                   ID_B    =>          $Servlet->params->{'ShippingFulfillmentMethodID'}
#
#--------------------------------------------------------------------------------------------------
# input        $Servlet
#                   | The Servlet object for the request
#                   | object
#==================================================================================================
sub _processFormData {
    my $self = shift ;
    my ( $Servlet ) = @_ ;

    my $Form = $Servlet->form ;

    # Get the value of the field to check if is in format <ID_A> or <ID_A>-<ID_B>

    my $LogisticsInfo = $Form->value( 'NewShippingFulfillmentTypeID' ) ;

    # To do the match we use a RegExp with grouping ( in order to catch the ID's )

    if( $LogisticsInfo =~ /^([0-9]+)(-([0-9]+))?$/x ) {

        # The ID match the format ...

        # The NewShippingFulfillmentTypeID is the ID_A part

        $Servlet->params->{'NewShippingFulfillmentTypeID'} = $1 ;

        if ( defined $3 ) {

            # Is a <ID_A>-<ID_B> format, so we have two ID's !!!

            # The ShippingFulfillmentMethodID is the ID_B

            $Servlet->params->{'ShippingFulfillmentMethodID'} = $3 ;
        }
    } else {
        $Form->executeFormError({
            'Name'  => 'NewShippingFulfillmentTypeID',
            'Reason'=> 'BAD_ID',
            'Value' => $LogisticsInfo,
        }) ;
    }

    return ;
}
#   END    _processFormData
#==================================================================================================

#==================================================================================================
# function     _getShippingFulfillmentMethod
# state        private
#--------------------------------------------------------------------------------------------------
# syntax       _getShippingFulfillmentMethod( $Shop ) ;
#--------------------------------------------------------------------------------------------------
# description  Returns a previous ShippingFulfillmentMethod after check if its available
#--------------------------------------------------------------------------------------------------
# input        $Servlet
#                   | The Servlet object for the request
#                   | object
# input        $ShippingFulfillmentMethodID
#                   | The ID for the existing Shipping Fulfillment Method
#                   | integer
# input        $ShippingFulfillmentType
#                   | The ID for the existing Shipping Fulfillment Type
#                   | integer
# return       $ShippingFulfillmentMethod
#                   | ShippingFulfillmentMethod
#                   | object
#==================================================================================================
sub _getShippingFulfillmentMethod {
    my $self = shift;
    my ( $Servlet, $ShippingFulfillmentMethodID, $ShippingFulfillmentType ) = @_;

    my $Shop = $Servlet->object ;

    my $ShippingFulfillmentObject =
        $Shop->loadObject(
            $ShippingFulfillmentMethodID,
            'ShippingFulfillmentMethod'
        ) ;

    # We have to check if the feature for this logistic is available

    $self->_checkFeatureAvailable( $Servlet, $ShippingFulfillmentType->alias, 'NewShippingFulfillmentTypeID' ) ;

    return $ShippingFulfillmentObject ;
}
#   END   _getShippingFulfillmentMethod
#==================================================================================================
#==================================================================================================
# function     _getShippingProviderMethod
# state        private
#--------------------------------------------------------------------------------------------------
# syntax       _getShippingProviderMethod( $Shop ) ;
#--------------------------------------------------------------------------------------------------
# description  Returns a previous _getShippingProviderMethod after check if its available
#--------------------------------------------------------------------------------------------------
# input        $Servlet
#                   | The Servlet object for the request
#                   | object
# input        $ShippingProviderMethodID
#                   | The ID for the existing Shipping Fulfillment Method
#                   | integer
# input        $ShippingFulfillmentType
#                   | The ID for the existing Shipping Fulfillment Type
#                   | integer
# return       $ShippingFulfillmentMethod
#                   | ShippingFulfillmentMethod
#                   | object
#==================================================================================================
sub _getShippingProviderMethod {
    my $self = shift;
    my ( $Servlet, $ShippingProviderMethodID, $ShippingFulfillmentType ) = @_;

    my $Shop = $Servlet->object ;

    my $ShippingFulfillmentObject =
        $Shop->loadObject(
            $ShippingProviderMethodID,
            'ShippingProviderMethod'
        ) ;

    # We have to check if the feature for this logistic is available

    $self->_checkFeatureAvailable( $Servlet, $ShippingFulfillmentType->alias, 'NewShippingFulfillmentTypeID' ) ;

    return $ShippingFulfillmentObject ;
}
#   END   _getShippingFulfillmentMethod

#==================================================================================================
# function     _newShippingFulfillmentMethod
# state        private
#--------------------------------------------------------------------------------------------------
# syntax       _newShippingFulfillmentMethod( $Servlet ) ;
#--------------------------------------------------------------------------------------------------
# description  From a given ShippingFulfillmentType ID we create a new ShippingFulfillmentMethod
#--------------------------------------------------------------------------------------------------
# input        $Servlet
#                   | The Servlet object for the request
#                   | object
# return       $NewShippingFulfillmentMethod
#                   | ShippingFulfillmentMethod
#                   | object
#==================================================================================================
sub _newShippingFulfillmentMethod {
    my $self = shift;
    my ( $Servlet ) = @_;

    # From the ID ShippingFulfillmentTypeID we create the Object

    my $ShippingFulfillmentType =
        LoadRootObject()->loadObject(
            $Servlet->params->{'NewShippingFulfillmentTypeID'},
            'ShippingFulfillmentType'
        ) ;

    # Its name is used for the NewAlias for the new ShippingFulfillmentTye

    my $ShippingFulfillmentTypeAlias = $ShippingFulfillmentType->alias ;

    # We have to check if the feature for this logistic is available

    $self->_checkFeatureAvailable( $Servlet, $ShippingFulfillmentTypeAlias, 'NewShippingFulfillmentTypeID' ) ;

    my $hInfo = { 'Alias' => $ShippingFulfillmentTypeAlias } ;

    my $Shop = $Servlet->object ;

    my $NewShippingFulfillmentMethod = $ShippingFulfillmentType->newShippingFulfillmentMethod( $Shop, $hInfo );

    $Servlet->params( 'ShippingFulfillmentMethodID', $NewShippingFulfillmentMethod->id ) ;

    return $NewShippingFulfillmentMethod ;
}
#   END   _newShippingFulfillmentMethod
#==================================================================================================
#==================================================================================================
# function     _newShippingFulfillmentMethod
# state        private
#--------------------------------------------------------------------------------------------------
# syntax       _newShippingFulfillmentMethod( $Servlet ) ;
#--------------------------------------------------------------------------------------------------
# description  From a given ShippingFulfillmentType ID we create a new ShippingFulfillmentMethod
#--------------------------------------------------------------------------------------------------
# input        $Servlet
#                   | The Servlet object for the request
#                   | object
# return       $NewShippingFulfillmentMethod
#                   | ShippingFulfillmentMethod
#                   | object
#==================================================================================================
sub _newShippingProviderMethod {
    my $self = shift;
    my ( $Servlet ) = @_;

    my $ShippingFulfillmentType =
        LoadRootObject()->loadObject(
            $Servlet->params->{'NewShippingFulfillmentTypeID'},
            'ShippingFulfillmentType'
        ) ;

    my $ShippingFulfillmentTypeAlias = $ShippingFulfillmentType->alias ;

    # We have to check if the feature for this logistic is available

    $self->_checkFeatureAvailable( $Servlet, $ShippingFulfillmentTypeAlias, 'NewShippingFulfillmentTypeID' ) ;

    my $hInfo = { 'Alias' => $ShippingFulfillmentTypeAlias } ;

    my $Shop = $Servlet->object ;

    my $NewShippingProviderMethod = $ShippingFulfillmentType->newShippingProviderMethod( $Shop, $hInfo );

    $Servlet->params( 'ShippingProviderMethodID', $NewShippingProviderMethod->id ) ;

    return $NewShippingProviderMethod ;
}
#   END   _newShippingFulfillmentMethod
#==================================================================================================

#==================================================================================================
# function     _checkFeatureAvailable
# state        private
#--------------------------------------------------------------------------------------------------
# syntax       _checkFeatureAvailable( $Shop, $FeatureName, $FormField ) ;
#--------------------------------------------------------------------------------------------------
# description  Verify if the feature ( ShippingFulfillmentType or ShippingType ) is available
#               If the feature isn't available an Error is thrown.
#--------------------------------------------------------------------------------------------------
# input        $Servlet
#                   | The Servlet object for the request
#                   | object
# input        $FeatureName
#                   | The name of the feature
#                   | string
# input        $formField
#                   | The name of field in the form
#                   | string
#==================================================================================================
sub _checkFeatureAvailable {
    my $self = shift;
    my ( $Servlet, $Feature, $FormField ) = @_;

    my $IsFeatureAvailable = 1 ;

# Disable the checking of the eval block if an error is thrown ...
# Some items ( like the ShippingType 'Free' ) has no features so if there isn't a feature we can
# consider it's enable.

## no critic qw(RequireExistsError)
    eval {
        my $Shop = $Servlet->object ;

        my $MaxValue = $Shop->featureMaxValue( $Feature ) ;

        my $CurrentValue = $Shop->featureCurrentValue( $Feature ) ;

        if ( $CurrentValue >= $MaxValue ) {
            $IsFeatureAvailable = 0 ;
        }
    } ;
## use critic

    if ( not $IsFeatureAvailable  ) {

        $Servlet->form->executeFormError({
            'Name'  => $FormField,
            'Reason'=> 'FEATURE_NOT_ALLOWED',
            'Value' => $Servlet->params->{$FormField},
        }) ;
    }

    return;
}
#   END   _checkFeatureAvailable
#==================================================================================================
#==================================================================================================
# function     _setLinkShippingFulfillmentMethod
# state        private
#--------------------------------------------------------------------------------------------------
# syntax       _setLinkShippingFulfillmentMethod( $ShippingMethod, $FulfillmentMethod ) ;
#--------------------------------------------------------------------------------------------------
# description  Once we have a ShippingMethod and a ShippingFulfillmentMethod the last step is
#               link the second to the first ...
#--------------------------------------------------------------------------------------------------
# input        $ShippingMethod
#                   | The Servlet object for the request
#                   | object
# input        $FulfillmentMethod
#                   | The Servlet object for the request
#                   | object
#==================================================================================================
sub _setLinkShippingFulfillmentMethod {
    my $self = shift ;
    my ( $ShippingMethod, $FulfillmentMethod ) = @_ ;

    if ( defined $FulfillmentMethod ) {

        $ShippingMethod->set( {
            'SelectedShippingFulfillmentMethod' => $FulfillmentMethod
        } ) ;
    }

    return ;
}
#   END    _setLinkShippingFulfillmentMethod
#==================================================================================================
#==================================================================================================
# function     _getSelectedShippingObjectMethod
# state        private
#--------------------------------------------------------------------------------------------------
# syntax       _getSelectedShippingObjectMethod( $ShippingMethod ) ;
#--------------------------------------------------------------------------------------------------
# description  Gets the Shiping provider Method or Shipping Fulfillment Method depending the kind
#               of Shipping Method
#--------------------------------------------------------------------------------------------------
# input        $ShippingMethod
#                   | Shipping MEthod
#                   | object
# return       $ShippingObjectMethod
#                   | ShippingProviderMethod or ShippingFulfillmentMethod
#                   | object
#==================================================================================================
sub _getSelectedShippingObjectMethod {
        my $self = shift ;
        my ( $ShippingMethod ) = @_ ;

        my $ShippingProviderMethod = $ShippingMethod->get('SelectedShippingProviderMethod');
        return defined($ShippingProviderMethod)
                      ? $ShippingProviderMethod
                      : $ShippingMethod->get('SelectedShippingFulfillmentMethod');
}

sub UnityViewShippingList {
    my $self = shift;
    my ($Servlet) = @_;

    # trigger hook to allow the adding of context sensitive apps
    TriggerHook('UI_Shop_UnityViewShippingList', { 'Servlet' => $Servlet });

    $self->SUPER::View($Servlet);
}
1;
