#========================================================================================
# package      DE_EPAGES::WebService::Servlet::Rest
# state        private
# base         DE_EPAGES::WebInterface::Servlet::BaseServlet
#----------------------------------------------------------------------------------------
# description  Servlet for handling RESTful web services.
#========================================================================================
package DE_EPAGES::WebService::Servlet::Rest;
use base DE_EPAGES::WebInterface::Servlet::BaseServlet;

use strict;
use feature 'switch';
use Encode qw( encode );
use HTTP::Status qw( HTTP_BAD_REQUEST HTTP_FORBIDDEN HTTP_NO_CONTENT HTTP_NOT_ACCEPTABLE HTTP_GONE
    HTTP_NOT_FOUND HTTP_METHOD_NOT_ALLOWED HTTP_INTERNAL_SERVER_ERROR HTTP_UNAUTHORIZED HTTP_CONFLICT );

use DE_EPAGES::Core::API::Error qw( Error ExistsError GetError );
use DE_EPAGES::Core::API::Log qw( GetLog LogError );
use DE_EPAGES::Core::API::PerlTools qw( UsePackage );
use DE_EPAGES::Core::API::TestParameter qw( TestObject );
use DE_EPAGES::Database::API::Config qw( DatabaseRedirect );
use DE_EPAGES::Database::API::Connection qw( RunOnStore );
use DE_EPAGES::Object::API::Factory qw( LoadObjectByGUID );
use DE_EPAGES::Permission::API::Permission qw( IsActionAllowed LogAction );
use DE_EPAGES::WebInterface::API::ApplicationServer qw( GetAppServer );
use DE_EPAGES::WebService::API::Authentication::Basic;
use DE_EPAGES::WebService::API::Authentication::OAuth;
use DE_EPAGES::WebService::API::Codec::JSON;
use DE_EPAGES::WebService::API::ResourceRegistry;

sub ErrorDefinition {
    return {
        Forbidden => 'User "#User" does not have permission to invoke "#Action" on "#Object"',
        UnknownAuthorizationMethod => 'Unknown authentication method "#Method"',
        Unauthorized => 'Authentication required to invoke "#Action" on "#Object"',
    };
}

#========================================================================================
# function     new
# state        public
#----------------------------------------------------------------------------------------
# syntax       $Servlet = new DE_EPAGES::WebService::Servlet::Rest->new;
#----------------------------------------------------------------------------------------
# description  Create a new Rest servlet object.
#----------------------------------------------------------------------------------------
# return       $Servlet | blessed object | object
#========================================================================================
sub new {
    my $class = shift;
    my $self = $class->SUPER::new(@_);
    $self->{Logger}   = GetLog();
    $self->{LogDebug} = $self->{Logger}->is_debug;
    return $self;
}

#========================================================================================
# function     handleRequest
# state        private
#----------------------------------------------------------------------------------------
# syntax       $RestServlet->handleRequest;
#----------------------------------------------------------------------------------------
# description  Decodes the request, dispatches to a REST resource and encodes the
#               response.
#========================================================================================
sub handleRequest {
    my $self = shift;

    my $Request = $self->request;
    $self->_logger->debug( "Request\n".$Request->as_string ) if ($self->_isLogDebug);

    local $self->{'SiteGUID'} = $self->_mapRequest($Request);

    my $StoreName = $self->storeName;
    my $RemoteAddr = GetAppServer()->serverVariables->{'REMOTE_ADDR'};
    my $Redirect = DatabaseRedirect($StoreName, undef, $RemoteAddr);
    Error('DatabaseRedirect', {
        'StoreName' => $StoreName,
    }) if $Redirect;

    RunOnStore( 'Store' => $StoreName, 'Sub' => sub {
        $self->response->header('X-Store', $StoreName);
        $self->_logger->debug('connected to store ', $StoreName);

        eval {
            my $hRequestContext = DE_EPAGES::WebService::API::ResourceRegistry->match( $self->site, $Request );
            $self->doAction($hRequestContext);
        };
        if (ExistsError()) {
            my $Error = GetError();
            given ($Error->code) {
                when ([qw( HandlerNotFound ResourceNotFound )]) {
                    $self->handleNotFound($Error);
                }
                when ('MethodNotImplemented') {
                    $self->handleMethodNotImplemented($Error);
                }
                when ('NotAcceptable') {
                    $self->handleNotAcceptable($Error);
                }
                when ([qw( UnknownAuthorizationMethod Forbidden Unauthorized ) ]) {
                    $self->handlePermissionDenied($Error);
                }
                when ([qw( BadRequest ) ]) {
                    $self->handleBadRequest($Error);
                }
                when ([qw( Conflict ) ]) {
                    $self->handleConflict($Error);
                }
                when ([qw( Gone ) ]) {
                    $self->handleGone($Error);
                }
                when ([qw( FeatureUnavailable ) ]) {
                    $self->handleFeatureUnavailable($Error);
                }
                default {
                    $self->_handleUnknownError($Error);
                }
            }
        }
    });

    $self->_logger->debug( "Response\n".$self->response->as_string."\n" ) if ($self->_isLogDebug);
}

sub _SiteGuidFromRequestHeader {
    my ($Request) = @_;

    my $SiteGUID = $Request->header('X-ePages-Site');
    my $RequestMethod = $Request->method();
    my $RequestURI = $Request->uri();

    die 'Missing header X-ePages-Site in request\n'.$RequestMethod.' '.$RequestURI unless $SiteGUID;
    return $SiteGUID;
}

sub _mapRequest {
    my $self = shift;
    my ($Request) = @_;

    if( my $RewriteURL = $Request->header('X-Rewrite-Url') ) { # using shop domain name (http(s)://shop-domain/api/...)
        # fix the request URL
        $Request->uri( $RewriteURL );
        return _SiteGuidFromRequestHeader($Request);
    }

    # EPG-34475: workaround for sites without domain name
    my $PathInfo = $self->serverVariables('PATH_INFO');
    if( $PathInfo =~ m|^/api| ) { # /epages/MyShop.rest/api
        $Request->uri->path($PathInfo);
        return _SiteGuidFromRequestHeader($Request);
    }
    elsif( $PathInfo =~ m|^/shops/(.*?)(/api.*)| ) { # /epages/Store.rest/shops/shop-guid/api
        my $SiteGUID = $1;
        my $NewPath = $2;
        $Request->uri->path($NewPath);
        return $SiteGUID;
    }
}

#========================================================================================
# function     handleError
# state        private
#----------------------------------------------------------------------------------------
# syntax       $RestServlet->handleError($Error);
#----------------------------------------------------------------------------------------
# description  Generic error handler.
#----------------------------------------------------------------------------------------
# input        $Error | Error object | DE_EPAGES::Core::API::Error
#========================================================================================
sub handleError {
    my $self = shift;
    my ($Error) = @_;

    GetLog( 'REST_SERVICE' )->debug( $Error->longMessage );

    $self->response->code( HTTP_INTERNAL_SERVER_ERROR );
    $self->_errorContent($Error);
}

sub _handleUnknownError {
    my $self = shift;
    my ($Error) = @_;

    LogError( $Error );
    $self->response->code( HTTP_INTERNAL_SERVER_ERROR );
    $self->_errorContent($Error);
}

#========================================================================================
# function     handleBadRequest
# state        private
#----------------------------------------------------------------------------------------
# syntax       $RestServlet->handleBadRequest($Error);
#----------------------------------------------------------------------------------------
# description  400 error handler.
#----------------------------------------------------------------------------------------
# input        $Error | Error object | DE_EPAGES::Core::API::Error
#========================================================================================
sub handleBadRequest {
    my $self = shift;
    my ($Error) = @_;

    $self->handleError($Error);
    $self->response->code( HTTP_BAD_REQUEST );
    return;
}

#========================================================================================
# function     handleConflict
# state        private
#----------------------------------------------------------------------------------------
# syntax       $RestServlet->handleConflict($Error);
#----------------------------------------------------------------------------------------
# description  409 error handler.
#----------------------------------------------------------------------------------------
# input        $Error | Error object | DE_EPAGES::Core::API::Error
#========================================================================================
sub handleConflict {
    my $self = shift;
    my ($Error) = @_;

    $self->handleError($Error);
    $self->response->code( HTTP_CONFLICT );
    return;
}

#========================================================================================
# function     handleGone
# state        private
#----------------------------------------------------------------------------------------
# syntax       $RestServlet->handleGone($Error);
#----------------------------------------------------------------------------------------
# description  410 error handler.
#----------------------------------------------------------------------------------------
# input        $Error | Error object | DE_EPAGES::Core::API::Error
#========================================================================================
sub handleGone {
    my $self = shift;
    my ($Error) = @_;

    $self->handleError($Error);
    $self->response->code( HTTP_GONE );
    return;
}

#========================================================================================
# function     handleFeatureUnavailale
# state        private
#----------------------------------------------------------------------------------------
# syntax       $RestServlet->handleFeatureUnavailale($Error);
#----------------------------------------------------------------------------------------
# description  403 error handler.
#----------------------------------------------------------------------------------------
# input        $Error | Error object | DE_EPAGES::Core::API::Error
#========================================================================================
sub handleFeatureUnavailable {
    my $self = shift;
    my ($Error) = @_;

    $self->handleError($Error);
    $self->response->code( HTTP_FORBIDDEN );
    return;
}

#========================================================================================
# function     handlePermissionDenied
# state        private
#----------------------------------------------------------------------------------------
# syntax       $RestServlet->handlePermissionDenied($Error);
#----------------------------------------------------------------------------------------
# description  401/403 error handler.
#----------------------------------------------------------------------------------------
# input        $Error | Error object | DE_EPAGES::Core::API::Error
#========================================================================================
sub handlePermissionDenied {
    my $self = shift;
    my ($Error) = @_;

    $self->handleError($Error);

    my $Response = $self->response;
    $Response->code( $Error->code eq 'Unauthorized' ? HTTP_UNAUTHORIZED : HTTP_FORBIDDEN );
    $Response->message( 'Unauthorized' );
    $Response->www_authenticate( 'Basic realm=\"epages REST service\"' );
    return;
}

#========================================================================================
# function     handleNotFound
# state        private
#----------------------------------------------------------------------------------------
# syntax       $RestServlet->handleNotFound($Error);
#----------------------------------------------------------------------------------------
# description  404 error handler.
#----------------------------------------------------------------------------------------
# input        $Error | Error object | DE_EPAGES::Core::API::Error
#========================================================================================
sub handleNotFound {
    my $self = shift;
    my ($Error) = @_;

    $self->handleError($Error);
    $self->response->code( HTTP_NOT_FOUND );
}

#========================================================================================
# function     handleMethodNotImplemented
# state        private
#----------------------------------------------------------------------------------------
# syntax       $RestServlet->handleMethodNotImplemented($Error);
#----------------------------------------------------------------------------------------
# description  405 error handler.
#----------------------------------------------------------------------------------------
# input        $Error | Error object | DE_EPAGES::Core::API::Error
#========================================================================================
sub handleMethodNotImplemented {
    my $self = shift;
    my ($Error) = @_;

    $self->handleError($Error);
    $self->response->code( HTTP_METHOD_NOT_ALLOWED );
}

#========================================================================================
# function     handleNotAcceptable
# state        private
#----------------------------------------------------------------------------------------
# syntax       $RestServlet->handleNotAcceptable($Error);
#----------------------------------------------------------------------------------------
# description  406 error handler.
#----------------------------------------------------------------------------------------
# input        $Error | Error object | DE_EPAGES::Core::API::Error
#========================================================================================
sub handleNotAcceptable {
    my $self = shift;
    my ($Error) = @_;

    $self->handleError($Error);
    $self->response->code( HTTP_NOT_ACCEPTABLE );
}

# set Response.Content from Error object
sub _errorContent {
    my $self = shift;
    my ($Error) = @_;

    # TODO: use content-type from Accept header
    my $Codec = DE_EPAGES::WebService::API::Codec::JSON->new;
    $Codec->encode($self->response, {
        message => $Error->shortMessage,
        code => $Error->code,
        guid => $Error->guid,
        hash => $Error->hash,
        data => $Error->vars()->{'Data'},
    });
}

#========================================================================================
# function     site
# state        public
#----------------------------------------------------------------------------------------
# syntax       $Site = $Servlet->site;
#----------------------------------------------------------------------------------------
# description  Gets the site object of the current request from the X-ePages-Site header.
#----------------------------------------------------------------------------------------
# return       $Site | site of the current request | object
#========================================================================================
sub site {
    my $self = shift;

    return LoadObjectByGUID($self->{'SiteGUID'});
}

#========================================================================================
# function     user
# state        public
#----------------------------------------------------------------------------------------
# syntax       $User = $Servlet->user;
#----------------------------------------------------------------------------------------
# description  Gets the authenticated user of the current request.
#----------------------------------------------------------------------------------------
# return       $User | user of the current request (undef if not authenticated) | object
#========================================================================================
sub user {
    my $self = shift;

    my $Request = $self->request;
    my $Authorization = $Request->authorization;
    unless ($Authorization) {
        $self->_logger->debug('No authorization header');
        return undef;
    }

    if( $Authorization =~ /^(\S+)/ ) {
        my $AuthorizationMethod = $1;
        given ($AuthorizationMethod) {
            when ('Basic') {
                return DE_EPAGES::WebService::API::Authentication::Basic->user( $Request, $self->site );
            }
            when ('OAuth') {
                return DE_EPAGES::WebService::API::Authentication::OAuth->user( $Request, $self->site );
            }
            default {
                Error('UnknownAuthorizationMethod', {
                    'Method' => $AuthorizationMethod
                });
            }
        }
    }
    else {
        Error('BadRequest', {
            'Reason' => 'InvalidAuthorizationHeader',
            'Header' => $Authorization,
        });
    }
}

#========================================================================================
# function     doAction
# state        public
#----------------------------------------------------------------------------------------
# syntax       $Servlet->doAction( $Object, $Action, $aParams );
# syntax       $Servlet->doAction( $Object, $Sub, $aParams );
#----------------------------------------------------------------------------------------
# description  Executes an action in response to a REST request.
#               1. Decodes the request data
#               2. Tests the permission
#               3. Executes the method that produces the response data
#               4. Encodes the response data
#----------------------------------------------------------------------------------------
# input        $Object | object to operate on | DE_EPAGES::Object::API::Object::Object
# input        $Action | action to execute | DE_EPAGES::Permission::API::Object::Action
# input        $Sub | function to execute | ref.code
# input        $aParams | parameters for the action function or code reference | ref.array
#========================================================================================
sub doAction {
    my $self = shift;
    my ($hRequestContext) = @_;

    my $Object = $hRequestContext->{'Object'};
    my $Action = $hRequestContext->{'Action'};
    my $aParams = $hRequestContext->{'Params'};
    my $RequestCodec = $hRequestContext->{'RequestCodec'};
    my $ResponseCodec = $hRequestContext->{'ResponseCodec'};
    my $User = $self->user;

    # decode the request data
    my $RequestData = defined $RequestCodec ? $RequestCodec->decode( $self->request ) : undef;

    my $ResponseData;
    if( ref $Action eq 'CODE' ) {
        $ResponseData = $Action->( @$aParams, $RequestData );
    }
    else {
        TestObject('Action', $Action, 'DE_EPAGES::Permission::API::Object::Action');

        # test permission or error
        $self->testPermission($Object, $User, $Action );

        my $Package = $Action->get('Package');
        my $Sub = $Action->get('FunctionName');
        die 'Action '.$Action->get('Path').' has no function name' unless defined $Sub;

        $self->_logger->debug('call action handler '.$Package.'::'.$Sub);
        UsePackage($Package);

        # log the action to the AUDITING log
        LogAction( 'Execute', $Object, $User, $Action );

        # execute action handler
        $ResponseData = $Package->$Sub( @$aParams, $RequestData );
    }

    # encode the response data
    if (defined $ResponseCodec && defined $ResponseData) {
        $ResponseCodec->encode( $self->response, $ResponseData );
    }
    else {
        $self->response->code(HTTP_NO_CONTENT);
        $self->response->content('');
    }

    return;
}

#========================================================================================
# function     testPermission
# state        public
#----------------------------------------------------------------------------------------
# syntax       $Resource->testPermission( $Object, $User, $Action);
#----------------------------------------------------------------------------------------
# description  Verifies if the current user has permission to execute a given action
#               on a given object. Throws an error if permission is denied.
#----------------------------------------------------------------------------------------
# input        $Object | object | object
# input        $User | user | object
# input        $Action | action object | object
#========================================================================================
sub testPermission {
    my $self = shift;
    my ($Object, $User, $Action) = @_;
    return if IsActionAllowed( $Object, $User, $Action );

    # throw the PermissionDenied error
    if (defined $User) {
        Error( 'Forbidden', {
            'User' => $User->pathString,
            'Object' => $Object->pathString,
            'Action' => $Action->pathString,
        });
    }
    else {
        Error( 'Unauthorized', {
            'Object' => $Object->pathString,
            'Action' => $Action->pathString,
        });
    }
}

#---------------------------------------------------------------------------
#  private helpers.
#---------------------------------------------------------------------------

sub _logger {
    return shift->{Logger};
}

sub _isLogDebug {
    return shift->{LogDebug};
}

1;
